@@ -16,8 +16,11 @@
 #    under the License.
 
 import keystonemiddleware.audit as audit_middleware
+from keystonemiddleware import auth_token
 from oslo_config import cfg
 import oslo_middleware.cors as cors_middleware
+from oslo_middleware import healthcheck
+from oslo_middleware import http_proxy_to_wsgi
 import osprofiler.web as osprofiler_web
 import pecan
 
@@ -25,8 +28,9 @@ from ironic.api import config
 from ironic.api.controllers import base
 from ironic.api import hooks
 from ironic.api import middleware
-from ironic.api.middleware import auth_token
+from ironic.api.middleware import auth_public_routes
 from ironic.api.middleware import json_ext
+from ironic.common import auth_basic
 from ironic.common import exception
 from ironic.conf import CONF
 
@@ -95,14 +99,35 @@ def setup_app(pecan_config=None, extra_hooks=None):
                 reason=e
             )
 
+    auth_middleware = None
     if CONF.auth_strategy == "keystone":
-        app = auth_token.AuthTokenMiddleware(
-            app, {"oslo_config_config": cfg.CONF},
+        auth_middleware = auth_token.AuthProtocol(
+            app, {"oslo_config_config": cfg.CONF})
+    elif CONF.auth_strategy == "http_basic":
+        auth_middleware = auth_basic.BasicAuthMiddleware(
+            app, cfg.CONF.http_basic_auth_user_file)
+
+    if auth_middleware:
+        app = auth_public_routes.AuthPublicRoutes(
+            app,
+            auth=auth_middleware,
             public_api_routes=pecan_config.app.acl_public_routes)
 
     if CONF.profiler.enabled:
         app = osprofiler_web.WsgiMiddleware(app)
 
+    # NOTE(pas-ha) this registers oslo_middleware.enable_proxy_headers_parsing
+    # option, when disabled (default) this is noop middleware
+    app = http_proxy_to_wsgi.HTTPProxyToWSGI(app, CONF)
+
+    # add in the healthcheck middleware if enabled
+    # NOTE(jroll) this is after the auth token middleware as we don't want auth
+    # in front of this, and WSGI works from the outside in. Requests to
+    # /healthcheck will be handled and returned before the auth middleware
+    # is reached.
+    if CONF.healthcheck.enabled:
+        app = healthcheck.Healthcheck(app, CONF)
+
     # Create a CORS wrapper, and attach ironic-specific defaults that must be
     # included in all CORS responses.
     app = IronicCORS(app, CONF)
