@@ -63,6 +63,8 @@ def config(token):
         # and behavior into place.
         'agent_token_required': True,
         'agent_md5_checksum_enable': CONF.agent.allow_md5_checksum,
+        'disable_deep_image_inspection': CONF.conductor.disable_deep_image_inspection,  # noqa
+        'permitted_image_formats': CONF.conductor.permitted_image_formats,
     }
 
 
@@ -290,8 +292,8 @@ DATA_VALIDATOR = args.schema({
         'inventory': {
             'type': 'object',
             'properties': {
-                'bmc_address': {'type': 'string'},
-                'bmc_v6address': {'type': 'string'},
+                'bmc_address': {'type': ['string', 'null']},
+                'bmc_v6address': {'type': ['string', 'null']},
                 'interfaces': {
                     'type': 'array',
                     'items': {
