@@ -14,7 +14,6 @@
 
 from http import client as http_client
 
-from ironic_lib import metrics_utils
 from oslo_utils import uuidutils
 from pecan import rest
 
@@ -27,6 +26,7 @@ from ironic.api import method
 from ironic.common import args
 from ironic.common import exception
 from ironic.common.i18n import _
+from ironic.common import metrics_utils
 from ironic.common import policy
 from ironic import objects
 
@@ -95,7 +95,7 @@ def convert_with_links(rpc_target, fields=None, sanitize=True):
     return target
 
 
-def list_convert_with_links(rpc_targets, limit, url=None, fields=None,
+def list_convert_with_links(rpc_targets, limit, url, fields=None,
                             detail=None, **kwargs):
     if detail:
         kwargs['detail'] = detail
