@@ -23,7 +23,6 @@ Based on pecan.middleware.errordocument
 import json
 
 from oslo_log import log
-import six
 
 from ironic.common.i18n import _
 
@@ -69,11 +68,9 @@ class ParsableErrorMiddleware(object):
 
         app_iter = self.app(environ, replacement_start_response)
         if (state['status_code'] // 100) not in (2, 3):
-            if six.PY3:
-                app_iter = [i.decode('utf-8') for i in app_iter]
+            app_iter = [i.decode('utf-8') for i in app_iter]
             body = [json.dumps({'error_message': '\n'.join(app_iter)})]
-            if six.PY3:
-                body = [item.encode('utf-8') for item in body]
+            body = [item.encode('utf-8') for item in body]
             state['headers'].append(('Content-Type', 'application/json'))
             state['headers'].append(('Content-Length', str(len(body[0]))))
         else:
