@@ -16,12 +16,13 @@
 
 import copy
 
+from oslo_log import log
 from oslo_serialization import jsonutils
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
 
 from ironic.common import exception
-
+from ironic.conf import CONF
 
 _IMAGE_ATTRIBUTES = ['size', 'disk_format', 'owner',
                      'container_format', 'checksum', 'id',
@@ -32,16 +33,31 @@ _IMAGE_ATTRIBUTES = ['size', 'disk_format', 'owner',
                      'os_hash_value']
 
 
+LOG = log.getLogger(__name__)
+
+
 def _extract_attributes(image):
     output = {}
+    # copies attributes from the openstacksdk Image object
+    # to a dictionary
     for attr in _IMAGE_ATTRIBUTES:
         output[attr] = getattr(image, attr, None)
 
-    output['properties'] = {}
+    # copy the properties over to start so that image properties
+    # are not nested in another properties key
+    output['properties'] = getattr(image, 'properties', {})
     output['schema'] = image.schema
 
-    for image_property in set(image) - set(_IMAGE_ATTRIBUTES):
-        output['properties'][image_property] = image[image_property]
+    # attributes already copied so we copy the rest into properties
+    copied = set(_IMAGE_ATTRIBUTES)
+    copied.add('properties')
+
+    for image_property in set(image) - copied:
+        # previously with glanceclient only set things
+        # were on the object that came back but the SDK
+        # defines every possible attribute so we need to filter
+        if image[image_property]:
+            output['properties'][image_property] = image[image_property]
 
     return output
 
@@ -107,18 +123,34 @@ def is_image_available(context, image):
     This check is needed in case Nova and Glance are deployed
     without authentication turned on.
     """
+    auth_token = getattr(context, 'auth_token', None)
+    image_visibility = getattr(image, 'visibility', None)
+    image_owner = getattr(image, 'owner', None)
+    image_id = getattr(image, 'id', 'unknown')
+    is_admin = 'admin' in getattr(context, 'roles', [])
+    project_id = getattr(context, 'project_id', None)
+    project = getattr(context, 'project', 'unknown')
     # The presence of an auth token implies this is an authenticated
     # request and we need not handle the noauth use-case.
-    if hasattr(context, 'auth_token') and context.auth_token:
+    if auth_token:
         # We return true here since we want the *user* request context to
         # be able to be used.
         return True
 
-    if getattr(image, 'visibility', None) == 'public':
+    if image_visibility == 'public':
         return True
 
-    return (context.project_id
-            and getattr(image, 'owner', None) == context.project_id)
+    if project_id and image_owner == project_id:
+        return True
+
+    if is_admin and CONF.ignore_project_check_for_admin_tasks:
+        return True
+
+    LOG.info(
+        'Access to %s owned by %s denied to requester %s',
+        image_id, image_owner, project
+    )
+    return False
 
 
 def is_image_active(image):
