@@ -21,6 +21,7 @@ from tooz import hashring
 
 from ironic.common import exception
 from ironic.common.i18n import _
+from ironic.common import utils
 from ironic.conf import CONF
 from ironic.db import api as dbapi
 
@@ -40,7 +41,7 @@ class HashRingManager(object):
     @property
     def ring(self):
         interval = CONF.hash_ring_reset_interval
-        limit = time.time() - interval
+        limit = time.monotonic() - interval
 
         if not self.cache:
             return self._load_hash_rings()
@@ -48,7 +49,11 @@ class HashRingManager(object):
         # Hot path, no lock. Using a local variable to avoid races with code
         # changing the class variable.
         hash_rings, updated_at = self.__class__._hash_rings
-        if hash_rings is not None and updated_at >= limit:
+        if (hash_rings is not None
+            and (updated_at >= limit
+                 or utils.is_ironic_using_sqlite())):
+            # Returning the hash ring for us, if it is still valid,
+            # or if we're using sqlite.
             return hash_rings
 
         with self._lock:
@@ -56,7 +61,7 @@ class HashRingManager(object):
             if hash_rings is None or updated_at < limit:
                 LOG.debug('Rebuilding cached hash rings')
                 hash_rings = self._load_hash_rings()
-                self.__class__._hash_rings = hash_rings, time.time()
+                self.__class__._hash_rings = hash_rings, time.monotonic()
                 LOG.debug('Finished rebuilding hash rings, available drivers '
                           'are %s', ', '.join(hash_rings))
             return hash_rings
@@ -68,7 +73,8 @@ class HashRingManager(object):
 
         for driver_name, hosts in d2c.items():
             rings[driver_name] = hashring.HashRing(
-                hosts, partitions=2 ** CONF.hash_partition_exponent)
+                hosts, partitions=2 ** CONF.hash_partition_exponent,
+                hash_function=CONF.hash_ring_algorithm)
 
         return rings
 
