@@ -23,16 +23,18 @@ import os
 import shutil
 import time
 
-from ironic_lib import qemu_img
 from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_utils import fileutils
+from oslo_utils.imageutils import format_inspector as image_format_inspector
 import pycdlib
 
+from ironic.common import checksum_utils
 from ironic.common import exception
 from ironic.common.glance_service import service_utils as glance_utils
 from ironic.common.i18n import _
 from ironic.common import image_service as service
+from ironic.common import qemu_img
 from ironic.common import utils
 from ironic.conf import CONF
 
@@ -99,7 +101,7 @@ def create_vfat_image(output_file, files_info=None, parameters=None,
              mounting, creating filesystem, copying files, etc.
     """
     try:
-        # TODO(sbaker): use ironic_lib.utils.dd when rootwrap has been removed
+        # TODO(sbaker): use utils.dd when rootwrap has been removed
         utils.execute('dd', 'if=/dev/zero', 'of=%s' % output_file, 'count=1',
                       'bs=%dKiB' % fs_size_kib)
     except processutils.ProcessExecutionError as e:
@@ -111,8 +113,7 @@ def create_vfat_image(output_file, files_info=None, parameters=None,
             # The label helps ramdisks to find the partition containing
             # the parameters (by using /dev/disk/by-label/ir-vfd-dev).
             # NOTE: FAT filesystem label can be up to 11 characters long.
-            # TODO(sbaker): use ironic_lib.utils.mkfs when rootwrap has been
-            #              removed
+            # TODO(sbaker): use utils.mkfs when rootwrap has been removed
             utils.execute('mkfs', '-t', 'vfat', '-n',
                           'ir-vfd-dev', output_file)
         except processutils.ProcessExecutionError as e:
@@ -363,7 +364,22 @@ def create_esp_image_for_uefi(
             raise exception.ImageCreationFailed(image_type='iso', error=e)
 
 
-def fetch_into(context, image_href, image_file):
+def fetch_into(context, image_href, image_file,
+               image_auth_data=None):
+    """Fetches image file contents into a file.
+
+    :param context: A context object.
+    :param image_href: The Image URL or reference to attempt to retrieve.
+    :param image_file: The file handler or file name to write the requested
+                       file contents to.
+    :param image_auth_data: Optional dictionary for credentials to be conveyed
+                            from the original task to the image download
+                            process, if required.
+    :returns: If a value is returned, that value was validated as the checksum.
+              Otherwise None indicating the process had been completed.
+    """
+    # TODO(TheJulia): We likely need to document all of the exceptions which
+    #                 can be raised by any of the various image services here.
     # TODO(vish): Improve context handling and add owner and auth data
     #             when it is added to glance.  Right now there is no
     #             auth checking in glance, so we assume that access was
@@ -375,6 +391,12 @@ def fetch_into(context, image_href, image_file):
                'image_href': image_href})
     start = time.time()
 
+    if image_service.is_auth_set_needed:
+        # Send a dictionary with username/password data,
+        # but send it in a dictionary since it fundimentally
+        # can differ dramatically by types.
+        image_service.set_image_auth(image_href, image_auth_data)
+
     if isinstance(image_file, str):
         with open(image_file, "wb") as image_file_obj:
             image_service.download(image_href, image_file_obj)
@@ -383,77 +405,152 @@ def fetch_into(context, image_href, image_file):
 
     LOG.debug("Image %(image_href)s downloaded in %(time).2f seconds.",
               {'image_href': image_href, 'time': time.time() - start})
+    if image_service.transfer_verified_checksum:
+        # TODO(TheJulia): The Glance Image service client does a
+        # transfer related check when it retrieves the file. We might want
+        # to shift the model some to do that upfront across most image
+        # services which are able to be used that way.
+
+        # We know, thanks to a value and not an exception, that
+        # we have a checksum which matches the transfer.
+        return image_service.transfer_verified_checksum
+    return None
 
 
-def fetch(context, image_href, path, force_raw=False):
+def _handle_zstd_compression(path):
+    zstd_comp = False
+    with open(path, 'rb') as comp_check:
+        # Check for zstd compression. Zstd has a variable window for streaming
+        # clients with transparent connections, and 128 byte blocks.
+        # Ultimately, requests can't support handling of such content without
+        # the zstandard library (bsd), but that is not available in global
+        # requirements. As such, and likely best complexity wise, if we find it
+        # we can handle it directly.
+        # https://github.com/facebook/zstd/blob/dev/doc/zstd_compression_format.md
+        # Ensure we're at the start of the file
+        comp_check.seek(0)
+        read = comp_check.read(4)
+        if read.startswith(b"\x28\xb5\x2f\xfd"):
+            zstd_comp = True
+
+    if zstd_comp and not CONF.conductor.disable_zstandard_decompression:
+        temp_path = path + '.zstd'
+        shutil.move(path, temp_path)
+        try:
+            utils.execute('zstd', '-d', '--rm', temp_path)
+        except OSError as e:
+            LOG.error('Failed to decompress a zstd compressed file: %s', e)
+            # Restore the downloaded file... We might want to fail the
+            # entire process.
+            shutil.move(temp_path, path)
+
+
+def fetch(context, image_href, path, force_raw=False,
+          checksum=None, checksum_algo=None,
+          image_auth_data=None):
     with fileutils.remove_path_on_error(path):
-        fetch_into(context, image_href, path)
+        transfer_checksum = fetch_into(context, image_href, path,
+                                       image_auth_data)
+        if (not transfer_checksum
+                and not CONF.conductor.disable_file_checksum
+                and checksum):
+            checksum_utils.validate_checksum(path, checksum, checksum_algo)
+
+    # Check and decompress zstd files, since python-requests realistically
+    # can't do it for us as-is. Also, some OCI container registry artifacts
+    # may generally just be zstd compressed, regardless if it is a raw file
+    # or a qcow2 file.
+    _handle_zstd_compression(path)
 
     if force_raw:
         image_to_raw(image_href, path, "%s.part" % path)
 
 
 def get_source_format(image_href, path):
-    data = qemu_img.image_info(path)
-
-    fmt = data.file_format
-    if fmt is None:
+    try:
+        img_format = image_format_inspector.detect_file_format(path)
+    except image_format_inspector.ImageFormatError as exc:
+        LOG.error("Parsing of the image %s failed: %s", image_href, exc)
         raise exception.ImageUnacceptable(
-            reason=_("'qemu-img info' parsing failed."),
+            reason=_("parsing of the image failed."),
             image_id=image_href)
-
-    backing_file = data.backing_file
-    if backing_file is not None:
+    if img_format is None:
+        LOG.error("Parsing of the image %s failed: format not recognized",
+                  image_href)
         raise exception.ImageUnacceptable(
-            image_id=image_href,
-            reason=_("fmt=%(fmt)s backed by: %(backing_file)s") %
-            {'fmt': fmt, 'backing_file': backing_file})
-
-    return fmt
+            reason=_("parsing of the image failed."),
+            image_id=image_href)
+    return str(img_format)
 
 
 def force_raw_will_convert(image_href, path_tmp):
     with fileutils.remove_path_on_error(path_tmp):
         fmt = get_source_format(image_href, path_tmp)
-    if fmt != "raw":
-        return True
-    return False
+    return fmt not in RAW_IMAGE_FORMATS
 
 
 def image_to_raw(image_href, path, path_tmp):
     with fileutils.remove_path_on_error(path_tmp):
-        fmt = get_source_format(image_href, path_tmp)
+        if not CONF.conductor.disable_deep_image_inspection:
+            fmt = safety_check_image(path_tmp)
 
-        if fmt != "raw":
+            if not image_format_permitted(fmt):
+                LOG.error("Security: The requested image %(image_href)s "
+                          "is of format image %(format)s and is not in "
+                          "the [conductor]permitted_image_formats list.",
+                          {'image_href': image_href,
+                           'format': fmt})
+                raise exception.InvalidImage()
+        else:
+            fmt = get_source_format(image_href, path)
+            LOG.warning("Security: Image safety checking has been disabled. "
+                        "This is unsafe operation. Attempting to continue "
+                        "the detected format %(img_fmt)s for %(path)s.",
+                        {'img_fmt': fmt,
+                         'path': path})
+
+        if fmt not in RAW_IMAGE_FORMATS and fmt != "iso":
+            # When the target format is NOT raw, we need to convert it.
+            # however, we don't need nor want to do that when we have
+            # an ISO image. If we have an ISO because it was requested,
+            # we have correctly fingerprinted it. Prior to proper
+            # image detection, we thought we had a raw image, and we
+            # would end up asking for a raw image to be made a raw image.
             staged = "%s.converted" % path
 
             utils.is_memory_insufficient(raise_if_fail=True)
             LOG.debug("%(image)s was %(format)s, converting to raw",
                       {'image': image_href, 'format': fmt})
             with fileutils.remove_path_on_error(staged):
-                qemu_img.convert_image(path_tmp, staged, 'raw')
+                qemu_img.convert_image(path_tmp, staged, 'raw',
+                                       source_format=fmt)
                 os.unlink(path_tmp)
-
-                data = qemu_img.image_info(staged)
-                if data.file_format != "raw":
+                new_fmt = get_source_format(image_href, staged)
+                if new_fmt not in RAW_IMAGE_FORMATS:
                     raise exception.ImageConvertFailed(
                         image_id=image_href,
                         reason=_("Converted to raw, but format is "
-                                 "now %s") % data.file_format)
+                                 "now %s") % new_fmt)
 
                 os.rename(staged, path)
         else:
             os.rename(path_tmp, path)
 
 
-def image_show(context, image_href, image_service=None):
+def image_show(context, image_href, image_service=None, image_auth_data=None):
     if image_service is None:
         image_service = service.get_image_service(image_href, context=context)
+    if image_service.is_auth_set_needed:
+        # We need to possibly authenticate, so we should attempt to do so.
+        image_service.set_image_auth(image_href, image_auth_data)
     return image_service.show(image_href)
 
 
-def download_size(context, image_href, image_service=None):
-    return image_show(context, image_href, image_service)['size']
+def download_size(context, image_href, image_service=None,
+                  image_auth_data=None):
+    return image_show(context, image_href,
+                      image_service=image_service,
+                      image_auth_data=image_auth_data)['size']
 
 
 def converted_size(path, estimate=False):
@@ -470,11 +567,11 @@ def converted_size(path, estimate=False):
         the original image scaled by the configuration value
         `raw_image_growth_factor`.
     """
-    data = qemu_img.image_info(path)
+    data = image_format_inspector.detect_file_format(path)
     if not estimate:
         return data.virtual_size
     growth_factor = CONF.raw_image_growth_factor
-    return int(min(data.disk_size * growth_factor, data.virtual_size))
+    return int(min(data.actual_size * growth_factor, data.virtual_size))
 
 
 def get_image_properties(context, image_href, properties="all"):
@@ -625,6 +722,11 @@ def is_whole_disk_image(ctx, instance_info):
 
         is_whole_disk_image = (not iproperties.get('kernel_id')
                                and not iproperties.get('ramdisk_id'))
+    elif service.is_container_registry_url(image_source):
+        # NOTE(theJulia): We can safely assume, at least outright,
+        # that all container images are whole disk images, unelss
+        # someone wants to add explicit support.
+        is_whole_disk_image = True
     else:
         # Non glance image ref
         if is_source_a_path(ctx, instance_info.get('image_source')):
@@ -787,3 +889,116 @@ def _get_deploy_iso_files(deploy_iso, mountdir):
     # present in deploy iso. This path varies for different OS vendors.
     # e_img_rel_path: is required by mkisofs to generate boot iso.
     return uefi_path_info, e_img_rel_path, grub_rel_path
+
+
+def __node_or_image_cache(node):
+    """A helper for logging to determine if image cache or node uuid."""
+    if not node:
+        return 'image cache'
+    else:
+        return node.uuid
+
+
+def safety_check_image(image_path, node=None):
+    """Performs a safety check on the supplied image.
+
+    This method triggers the image format inspector's to both identify the
+    type of the supplied file and safety check logic to identify if there
+    are any known unsafe features being leveraged, and return the detected
+    file format in the form of a string for the caller.
+
+    :param image_path: A fully qualified path to an image which needs to
+                       be evaluated for safety.
+    :param node: A Node object, optional. When supplied logging indicates the
+                 node which triggered this issue, but the node is not
+                 available in all invocation cases.
+    :returns: a string representing the the image type which is used.
+    :raises: InvalidImage when the supplied image is detected as unsafe,
+             or the image format inspector has failed to parse the supplied
+             image's contents.
+    """
+    id_string = __node_or_image_cache(node)
+    try:
+        img_class = image_format_inspector.detect_file_format(image_path)
+        if img_class is None:
+            LOG.error("Security: The requested user image for the "
+                      "deployment node %(node)s does not match any known "
+                      "format",
+                      {'node': id_string})
+            raise exception.InvalidImage()
+        img_class.safety_check()
+        image_format_name = str(img_class)
+    except image_format_inspector.ImageFormatError as exc:
+        LOG.error("Security: The requested user image for the "
+                  "deployment node %(node)s failed to be able "
+                  "to be parsed by the image format checker: %(exc)s",
+                  {'node': id_string, 'exc': exc})
+        raise exception.InvalidImage()
+    except image_format_inspector.SafetyCheckFailed as exc:
+        LOG.error("Security: The requested image for "
+                  "deployment of node %(node)s fails safety sanity "
+                  "checking: %(exc)s",
+                  {'node': id_string, 'exc': exc})
+        raise exception.InvalidImage()
+    return image_format_name
+
+
+RAW_IMAGE_FORMATS = {'raw', 'gpt'}  # gpt is a whole-disk image
+
+
+def image_format_permitted(img_format):
+    permitted = set(CONF.conductor.permitted_image_formats)
+    if 'raw' in permitted:
+        permitted.update(RAW_IMAGE_FORMATS)
+    return img_format in permitted
+
+
+def image_format_matches(actual_format, expected_format):
+    if expected_format in ['ari', 'aki']:
+        # In this case, we have an ari or aki, meaning we're pulling
+        # down a kernel/ramdisk, and this is rooted in a misunderstanding.
+        # They should be raw. The detector should be detecting this *as*
+        # raw anyway, so the data just mismatches from a common
+        # misunderstanding, and that is okay in this case as they are not
+        # passed to qemu-img.
+        # TODO(TheJulia): Add a log entry to warn here at some point in
+        # the future as we begin to shift the perception around this.
+        # See: https://bugs.launchpad.net/ironic/+bug/2074090
+        return True
+    if expected_format == 'raw' and actual_format in RAW_IMAGE_FORMATS:
+        return True
+    return expected_format == actual_format
+
+
+def check_if_image_format_is_permitted(img_format,
+                                       expected_format=None,
+                                       node=None):
+    """Checks image format consistency.
+
+    :params img_format: The determined image format by name.
+    :params expected_format: Optional, the expected format based upon
+        supplied configuration values.
+    :params node: A node object or None implying image cache.
+    :raises: InvalidImage if the requested image format is not permitted
+             by configuration, or the expected_format does not match the
+             determined format.
+    """
+
+    id_string = __node_or_image_cache(node)
+    if not image_format_permitted(img_format):
+        LOG.error("Security: The requested deploy image for node %(node)s "
+                  "is of format image %(format)s and is not in the "
+                  "[conductor]permitted_image_formats list.",
+                  {'node': id_string,
+                   'format': img_format})
+        raise exception.InvalidImage()
+    if (expected_format is not None
+            and not image_format_matches(img_format, expected_format)):
+        LOG.error("Security: The requested deploy image for node %(node)s "
+                  "has a format (%(format)s) which does not match the "
+                  "expected image format (%(expected)s) based upon "
+                  "supplied or retrieved information.",
+                  {'node': id_string,
+                   'format': img_format,
+                   'expected': expected_format})
+        raise exception.InvalidImage()
