@@ -15,20 +15,22 @@
 This client is compatible with any JSON RPC 2.0 implementation, including ours.
 """
 
+import logging
+
 from oslo_config import cfg
-from oslo_log import log
 from oslo_utils import importutils
+from oslo_utils import netutils
 from oslo_utils import strutils
 from oslo_utils import uuidutils
 
 from ironic.common import exception
 from ironic.common.i18n import _
-from ironic.common import json_rpc
 from ironic.common import keystone
+from ironic.conf import json_rpc
 
 
 CONF = cfg.CONF
-LOG = log.getLogger(__name__)
+LOG = logging.getLogger(__name__)
 _SESSION = None
 
 
@@ -66,6 +68,11 @@ def _get_session():
 class Client(object):
     """JSON RPC client with ironic exception handling."""
 
+    allowed_exception_namespaces = [
+        "ironic.common.exception.",
+        "ironic_inspector.utils.",
+    ]
+
     def __init__(self, serializer, version_cap=None):
         self.serializer = serializer
         self.version_cap = version_cap
@@ -74,19 +81,42 @@ class Client(object):
         return _can_send_version(version, self.version_cap)
 
     def prepare(self, topic, version=None):
+        """Prepare the client to transmit a request.
+
+        :param topic: Topic which is being addressed. Typically this
+                      is the hostname of the remote json-rpc service.
+        :param version: The RPC API version to utilize.
+        """
+
         host = topic.split('.', 1)[1]
-        return _CallContext(host, self.serializer, version=version,
-                            version_cap=self.version_cap)
+        host, port = netutils.parse_host_port(host)
+        return _CallContext(
+            host, self.serializer, version=version,
+            version_cap=self.version_cap,
+            allowed_exception_namespaces=self.allowed_exception_namespaces,
+            port=port)
 
 
 class _CallContext(object):
     """Wrapper object for compatibility with oslo.messaging API."""
 
-    def __init__(self, host, serializer, version=None, version_cap=None):
+    def __init__(self, host, serializer, version=None, version_cap=None,
+                 allowed_exception_namespaces=(), port=None):
+        if not port:
+            self.port = CONF.json_rpc.port
+        else:
+            self.port = int(port)
         self.host = host
         self.serializer = serializer
         self.version = version
         self.version_cap = version_cap
+        self.allowed_exception_namespaces = allowed_exception_namespaces
+
+    def _is_known_exception(self, class_name):
+        for ns in self.allowed_exception_namespaces:
+            if class_name.startswith(ns):
+                return True
+        return False
 
     def _handle_error(self, error):
         if not error:
@@ -100,7 +130,7 @@ class _CallContext(object):
             raise exception.IronicException(
                 _("Unexpected error raised by RPC"))
         else:
-            if not cls.startswith('ironic.common.exception.'):
+            if not self._is_known_exception(cls):
                 # NOTE(dtantsur): protect against arbitrary code execution
                 LOG.error("Unexpected error from RPC: %s", error)
                 raise exception.IronicException(
@@ -164,17 +194,24 @@ class _CallContext(object):
             "params": params,
         }
         if not cast:
-            body['id'] = context.request_id or uuidutils.generate_uuid()
+            body['id'] = (getattr(context, 'request_id', None)
+                          or uuidutils.generate_uuid())
 
-        LOG.debug("RPC %s with %s", method, strutils.mask_dict_password(body))
         scheme = 'http'
         if CONF.json_rpc.use_ssl:
             scheme = 'https'
-        url = '%s://%s:%d' % (scheme, self.host, CONF.json_rpc.port)
-        result = _get_session().post(url, json=body)
-        LOG.debug('RPC %s returned %s', method,
+        url = '%s://%s:%d' % (scheme,
+                              netutils.escape_ipv6(self.host),
+                              self.port)
+        LOG.debug("RPC %s to %s with %s", method, url,
+                  strutils.mask_dict_password(body))
+        try:
+            result = _get_session().post(url, json=body)
+        except Exception as exc:
+            LOG.debug('RPC %s to %s failed with %s', method, url, exc)
+            raise
+        LOG.debug('RPC %s to %s returned %s', method, url,
                   strutils.mask_password(result.text or '<None>'))
-
         if not cast:
             result = result.json()
             self._handle_error(result.get('error'))
