@@ -55,6 +55,8 @@ def do_node_service(task, service_steps=None, disable_ramdisk=False):
                                   disable_ramdisk)
     task.node.save()
 
+    utils.node_update_cache(task)
+
     # Allow the deploy driver to set up the ramdisk again (necessary for IPA)
     try:
         if not disable_ramdisk:
@@ -131,6 +133,10 @@ def do_next_service_step(task, step_index, disable_ramdisk=None):
         eocn = step.get('execute_on_child_nodes', False)
         result = None
         try:
+            if async_steps.SERVICING_POLLING in node.driver_internal_info:
+                # We're going to execute a new step, we should delete any
+                # older/out of date option.
+                node.del_driver_internal_info(async_steps.SERVICING_POLLING)
             if not eocn:
                 LOG.info('Executing %(step)s on node %(node)s',
                          {'step': step, 'node': node.uuid})
@@ -228,6 +234,7 @@ def _tear_down_node_service(task, disable_ramdisk):
             return utils.servicing_error_handler(task, msg,
                                                  traceback=True,
                                                  tear_down_service=False)
+    utils.node_update_cache(task)
     LOG.info('Node %s service complete.', task.node.uuid)
     task.process_event('done')
 
