@@ -45,12 +45,14 @@ from ironic.conf import metrics
 from ironic.conf import molds
 from ironic.conf import neutron
 from ironic.conf import nova
+from ironic.conf import oci
 from ironic.conf import pxe
 from ironic.conf import redfish
 from ironic.conf import sensor_data
 from ironic.conf import service_catalog
 from ironic.conf import snmp
 from ironic.conf import swift
+from ironic.conf import vnc
 
 CONF = cfg.CONF
 
@@ -84,9 +86,11 @@ metrics.register_opts(CONF)
 molds.register_opts(CONF)
 neutron.register_opts(CONF)
 nova.register_opts(CONF)
+oci.register_opts(CONF)
 pxe.register_opts(CONF)
 redfish.register_opts(CONF)
 sensor_data.register_opts(CONF)
 service_catalog.register_opts(CONF)
 snmp.register_opts(CONF)
 swift.register_opts(CONF)
+vnc.register_opts(CONF)
