@@ -88,7 +88,8 @@ opts = [
     cfg.StrOpt('default_hooks',
                default='ramdisk-error,validate-interfaces,ports,architecture',
                help=_('A comma-separated lists of inspection hooks that are '
-                      'run by default. In most cases, the operators will not '
+                      'run by default for the "agent" inspection interface. '
+                      'In most cases, the operators will not '
                       'modify this. The default (somewhat conservative) hooks '
                       'will raise an exception in case the ramdisk reports an '
                       'error, validate interfaces in the inventory, create '
@@ -96,7 +97,8 @@ opts = [
     cfg.StrOpt('hooks',
                default='$default_hooks',
                help=_('Comma-separated list of enabled hooks for processing '
-                      'pipeline. The default for this is $default_hooks. '
+                      'pipeline for the "agent" inspection interface. '
+                      'The default for this is $default_hooks. '
                       'Hooks can be added before or after the defaults '
                       'like this: "prehook,$default_hooks,posthook".')),
     cfg.StrOpt('known_accelerators',
@@ -155,6 +157,10 @@ discovery_opts = [
                mutable=True,
                help=_("The default driver to use for newly enrolled nodes. "
                       "Must be set when enabling auto-discovery.")),
+    cfg.StrOpt('inspection_scope',
+               default=None,
+               help=_("The default inspection scope for nodes enrolled via "
+                      "auto-discovery.")),
 ]
 
 pxe_filter_opts = [
@@ -171,7 +177,18 @@ pxe_filter_opts = [
     cfg.IntOpt('sync_period',
                default=45, mutable=True,
                help=_("Period (in seconds) between synchronizing the state "
-                      "if dnsmasq with the database.")),
+                      "of dnsmasq with the database.")),
+]
+
+inspection_rule_opts = [
+    cfg.StrOpt('built_in_rules',
+               mutable=True,
+               help=_("Path to YAML file of built-in inspection rules.")),
+    cfg.StrOpt('mask_secrets',
+               default='always',
+               choices=['always', 'never', 'sensitive'],
+               help=_("Whether to mask secrets in the node information "
+                      "passed to the rules."))
 ]
 
 
@@ -179,6 +196,7 @@ def register_opts(conf):
     conf.register_opts(opts, group='inspector')
     conf.register_opts(discovery_opts, group='auto_discovery')
     conf.register_opts(pxe_filter_opts, group='pxe_filter')
+    conf.register_opts(inspection_rule_opts, group='inspection_rules')
     auth.register_auth_opts(conf, 'inspector',
                             service_type='baremetal-introspection')
 
