@@ -20,18 +20,15 @@ Abstract base class for dhcp providers.
 import abc
 
 from oslo_log import log as logging
-import six
 
 LOG = logging.getLogger(__name__)
 
 
-@six.add_metaclass(abc.ABCMeta)
-class BaseDHCP(object):
+class BaseDHCP(object, metaclass=abc.ABCMeta):
     """Base class for DHCP provider APIs."""
 
     @abc.abstractmethod
-    def update_port_dhcp_opts(self, port_id, dhcp_options, token=None,
-                              context=None):
+    def update_port_dhcp_opts(self, port_id, dhcp_options, context=None):
         """Update one or more DHCP options on the specified port.
 
         :param port_id: designate which port these attributes
@@ -41,19 +38,15 @@ class BaseDHCP(object):
                              ::
 
                               [{'opt_name': '67',
-                                'opt_value': 'pxelinux.0'},
+                                'opt_value': 'pxelinux.0',
+                                'ip_version': 4},
                                {'opt_name': '66',
-                                'opt_value': '123.123.123.456'}]
-        :param token: An optional authentication token. Deprecated, use context
+                                'opt_value': '123.123.123.456',
+                                'ip_version': 4}]
         :param context: request context
         :type context: ironic.common.context.RequestContext
         :raises: FailedToUpdateDHCPOptOnPort
         """
-        # TODO(pas-ha) ignore token arg in Rocky
-        if token:
-            LOG.warning("Using the 'token' argument is deprecated, "
-                        "use the 'context' argument to pass the "
-                        "full request context instead.")
 
     @abc.abstractmethod
     def update_dhcp_opts(self, task, options, vifs=None):
@@ -65,9 +58,11 @@ class BaseDHCP(object):
                         ::
 
                          [{'opt_name': '67',
-                           'opt_value': 'pxelinux.0'},
+                           'opt_value': 'pxelinux.0',
+                           'ip_version': 4},
                           {'opt_name': '66',
-                           'opt_value': '123.123.123.456'}]
+                           'opt_value': '123.123.123.456',
+                           'ip_version': 4}]
 
         :param vifs: A dict with keys 'ports' and 'portgroups' and
             dicts as values. Each dict has key/value pairs of the form
@@ -100,3 +95,14 @@ class BaseDHCP(object):
         :raises: FailedToCleanDHCPOpts
         """
         pass
+
+    def supports_ipxe_tag(self):
+        """Whether the provider will correctly apply the 'ipxe' tag.
+
+        When iPXE makes a DHCP request, does this provider support adding
+        the tag `ipxe` or `ipxe6` (for IPv6). When the provider returns True,
+        options can be added which filter on these tags.
+
+        :returns: True when the driver supports tagging iPXE DHCP requests
+        """
+        return False
