@@ -19,8 +19,7 @@ from ironic.dhcp import base
 class NoneDHCPApi(base.BaseDHCP):
     """No-op DHCP API."""
 
-    def update_port_dhcp_opts(self, port_id, dhcp_options, token=None,
-                              context=None):
+    def update_port_dhcp_opts(self, port_id, dhcp_options, context=None):
         pass
 
     def update_dhcp_opts(self, task, options, vifs=None):
