@@ -12,9 +12,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import base64
 from urllib import parse as urlparse
 
-from ironic_lib import metrics_utils
 from oslo_log import log
 from oslo_utils import strutils
 from oslo_utils import units
@@ -22,11 +22,14 @@ import tenacity
 
 from ironic.common import async_steps
 from ironic.common import boot_devices
+from ironic.common import boot_modes
 from ironic.common import exception
 from ironic.common.glance_service import service_utils
 from ironic.common.i18n import _
 from ironic.common import image_service
 from ironic.common import images
+from ironic.common import metrics_utils
+from ironic.common import oci_registry as oci
 from ironic.common import raid
 from ironic.common import states
 from ironic.common import utils
@@ -248,6 +251,51 @@ def soft_power_off(task, client=None):
         manager_utils.node_power_action(task, states.POWER_OFF)
 
 
+def set_boot_to_disk(task, target_boot_mode=None):
+    """Boot a node to disk.
+
+    This is a helper method to reduce duplication of code around
+    handling vendor specifics for setting boot modes between multiple
+    deployment interfaces inside of Ironic.
+
+    :param task: A Taskmanager object.
+    :param target_boot_mode: The target boot_mode, defaults to UEFI.
+    """
+    if not target_boot_mode:
+        target_boot_mode = boot_modes.UEFI
+    node = task.node
+    try:
+        persistent = True
+        # NOTE(TheJulia): We *really* only should be doing this in bios
+        # boot mode. In UEFI this might just get disregarded, or cause
+        # issues/failures.
+        if node.driver_info.get('force_persistent_boot_device',
+                                'Default') == 'Never':
+            persistent = False
+
+        vendor = task.node.properties.get('vendor', None)
+        if not (vendor and vendor.lower() == 'lenovo'
+                and target_boot_mode == 'uefi'):
+            # Lenovo hardware is modeled on a "just update"
+            # UEFI nvram model of use, and if multiple actions
+            # get requested, you can end up in cases where NVRAM
+            # changes are deleted as the host "restores" to the
+            # backup. For more information see
+            # https://bugs.launchpad.net/ironic/+bug/2053064
+            # NOTE(TheJulia): We likely just need to do this with
+            # all hosts in uefi mode, but libvirt VMs don't handle
+            # nvram only changes *and* this pattern is known to generally
+            # work for Ironic operators.
+            deploy_utils.try_set_boot_device(task, boot_devices.DISK,
+                                             persistent=persistent)
+    except Exception as e:
+        msg = (_("Failed to change the boot device to %(boot_dev)s "
+                 "when deploying node %(node)s: %(error)s") %
+               {'boot_dev': boot_devices.DISK, 'node': node.uuid,
+                'error': e})
+        agent_base.log_and_raise_deployment_error(task, msg, exc=e)
+
+
 class CustomAgentDeploy(agent_base.AgentBaseMixin,
                         agent_base.HeartbeatMixin,
                         agent_base.AgentOobStepsMixin,
@@ -412,7 +460,11 @@ class CustomAgentDeploy(agent_base.AgentBaseMixin,
 
         client = agent_client.get_client(task)
         try:
-            if not can_power_on:
+            if node.disable_power_off:
+                LOG.info("Node %s does not support power off, locking "
+                         "down the agent", node.uuid)
+                client.lockdown(node)
+            elif not can_power_on:
                 LOG.info('Power interface of node %s does not support '
                          'power on, using reboot to switch to the instance',
                          node.uuid)
@@ -486,7 +538,7 @@ class CustomAgentDeploy(agent_base.AgentBaseMixin,
                 LOG.debug('The agent for node %(node)s has recently checked '
                           'in, and the node power will remain unmodified.',
                           {'node': task.node.uuid})
-            else:
+            elif not node.disable_power_off:
                 # Powering off node to setup networking for port and
                 # ensure that the state is reset if it is inadvertently
                 # on for any unknown reason.
@@ -607,7 +659,8 @@ class AgentDeploy(CustomAgentDeploy):
         # NOTE(dtantsur): glance images contain a checksum; for file images we
         # will recalculate the checksum anyway.
         if (not service_utils.is_glance_image(image_source)
-                and not image_source.startswith('file://')):
+                and not image_source.startswith('file://')
+                and not image_source.startswith('oci://')):
 
             def _raise_missing_checksum_exception(node):
                 raise exception.MissingParameterValue(_(
@@ -677,6 +730,11 @@ class AgentDeploy(CustomAgentDeploy):
             image_info['os_hash_value'] = node.instance_info[
                 'image_os_hash_value']
 
+        if node.instance_info.get('image_request_authorization_secret'):
+            ah = node.instance_info.get('image_request_authorization_secret')
+            ah = base64.standard_b64encode(ah.encode())
+            image_info['image_request_authorization'] = ah
+
         proxies = {}
         for scheme in ('http', 'https'):
             proxy_param = 'image_%s_proxy' % scheme
@@ -905,40 +963,94 @@ class AgentDeploy(CustomAgentDeploy):
                         'error': agent_client.get_command_error(result)})
                 agent_base.log_and_raise_deployment_error(task, msg)
 
-        try:
-            persistent = True
-            # NOTE(TheJulia): We *really* only should be doing this in bios
-            # boot mode. In UEFI this might just get disregarded, or cause
-            # issues/failures.
-            if node.driver_info.get('force_persistent_boot_device',
-                                    'Default') == 'Never':
-                persistent = False
-
-            vendor = task.node.properties.get('vendor', None)
-            if not (vendor and vendor.lower() == 'lenovo'
-                    and target_boot_mode == 'uefi'):
-                # Lenovo hardware is modeled on a "just update"
-                # UEFI nvram model of use, and if multiple actions
-                # get requested, you can end up in cases where NVRAM
-                # changes are deleted as the host "restores" to the
-                # backup. For more information see
-                # https://bugs.launchpad.net/ironic/+bug/2053064
-                # NOTE(TheJulia): We likely just need to do this with
-                # all hosts in uefi mode, but libvirt VMs don't handle
-                # nvram only changes *and* this pattern is known to generally
-                # work for Ironic operators.
-                deploy_utils.try_set_boot_device(task, boot_devices.DISK,
-                                                 persistent=persistent)
-        except Exception as e:
-            msg = (_("Failed to change the boot device to %(boot_dev)s "
-                     "when deploying node %(node)s: %(error)s") %
-                   {'boot_dev': boot_devices.DISK, 'node': node.uuid,
-                    'error': e})
-            agent_base.log_and_raise_deployment_error(task, msg, exc=e)
-
+        set_boot_to_disk(task, target_boot_mode)
         LOG.info('Local boot successfully configured for node %s', node.uuid)
 
 
+class BootcAgentDeploy(CustomAgentDeploy):
+    """Interface for deploy-related actions."""
+
+    @METRICS.timer('AgentBootcDeploy.validate')
+    def validate(self, task):
+        """Validate the driver-specific Node deployment info.
+
+        This method validates whether the properties of the supplied node
+        contain the required information for this driver to deploy images to
+        the node.
+
+        :param task: a TaskManager instance
+        :raises: MissingParameterValue, if any of the required parameters are
+            missing.
+        :raises: InvalidParameterValue, if any of the parameters have invalid
+            value.
+        """
+        super().validate(task)
+
+        node = task.node
+
+        image_source = node.instance_info.get('image_source')
+        if not image_source or not image_source.startswith('oci://'):
+            raise exception.InvalidImageRef(image_href=image_source)
+
+    @METRICS.timer('AgentBootcDeploy.execute_bootc_install')
+    @base.deploy_step(priority=80)
+    @task_manager.require_exclusive_lock
+    def execute_bootc_install(self, task):
+        node = task.node
+        image_source = node.instance_info.get('image_source')
+        # FIXME(TheJulia): We likely, either need to grab/collect creds
+        # and pass them along in the step call, or initialize the client.
+        # bootc runs in the target container as well, so ... hmmm
+        configdrive = manager_utils.get_configdrive_image(node)
+
+        img_auth = image_service.get_image_service_auth_override(task.node)
+
+        if not img_auth:
+            fqdn = urlparse.urlparse(image_source).netloc
+            img_auth = oci.RegistrySessionHelper.get_token_from_config(
+                fqdn)
+        else:
+            # Internally, image data is a username and password, and we
+            # only currently support pull secrets which are just transmitted
+            # via the password value.
+            img_auth = img_auth.get('password')
+        if img_auth:
+            # This is not encryption, but obfustication.
+            img_auth = base64.standard_b64encode(img_auth.encode())
+        # Now switch into the corresponding in-band deploy step and let the
+        # result be polled normally.
+        new_step = {'interface': 'deploy',
+                    'step': 'execute_bootc_install',
+                    'args': {'image_source': image_source,
+                             'configdrive': configdrive,
+                             'oci_pull_secret': img_auth}}
+        client = agent_client.get_client(task)
+        return agent_base.execute_step(task, new_step, 'deploy',
+                                       client=client)
+
+    @METRICS.timer('AgentBootcDeploy.set_boot_to_disk')
+    @base.deploy_step(priority=60)
+    @task_manager.require_exclusive_lock
+    def set_boot_to_disk(self, task):
+        """Sets the node to boot from disk.
+
+        In some cases, other steps may handle aspects like bootloaders
+        and UEFI NVRAM entries required to boot. That leaves one last
+        aspect, resetting the node to boot from disk.
+
+        This primarily exists for compatibility reasons of flow
+        for Ironic, but we know some BMCs *really* need to be
+        still told to boot from disk. The exception to this is
+        Lenovo hardware, where we skip the action because it
+        can create a UEFI NVRAM update failure case, which
+        reverts the NVRAM state to "last known good configuration".
+
+        :param task: A Taskmanager object.
+        """
+        # Call the helper to de-duplicate code.
+        set_boot_to_disk(task)
+
+
 class AgentRAID(base.RAIDInterface):
     """Implementation of RAIDInterface which uses agent ramdisk."""
 
@@ -1136,24 +1248,23 @@ class AgentRescue(base.RescueInterface):
         :raises: any boot interface's prepare_ramdisk exceptions.
         :returns: Returns states.RESCUEWAIT
         """
-        manager_utils.node_power_action(task, states.POWER_OFF)
-        # NOTE(TheJulia): Revealing that the power is off at any time can
-        # cause external power sync to decide that the node must be off.
-        # This may result in a post-rescued instance being turned off
-        # unexpectedly after rescue has started.
-        # TODO(TheJulia): Once we have power/state callbacks to nova,
-        # the reset of the power_state can be removed.
-        task.node.power_state = states.POWER_ON
-        task.node.save()
+        with driver_utils.power_off_and_on(task):
+            # NOTE(TheJulia): Revealing that the power is off at any time can
+            # cause external power sync to decide that the node must be off.
+            # This may result in a post-rescued instance being turned off
+            # unexpectedly after rescue has started.
+            # TODO(TheJulia): Once we have power/state callbacks to nova,
+            # the reset of the power_state can be removed.
+            task.node.power_state = states.POWER_ON
+            task.node.save()
 
-        task.driver.boot.clean_up_instance(task)
-        with manager_utils.power_state_for_network_configuration(task):
-            task.driver.network.unconfigure_tenant_networks(task)
-            task.driver.network.add_rescuing_network(task)
-        if CONF.agent.manage_agent_boot:
-            # prepare_ramdisk will set the boot device
-            deploy_utils.prepare_agent_boot(task)
-        manager_utils.node_power_action(task, states.POWER_ON)
+            task.driver.boot.clean_up_instance(task)
+            with manager_utils.power_state_for_network_configuration(task):
+                task.driver.network.unconfigure_tenant_networks(task)
+                task.driver.network.add_rescuing_network(task)
+            if CONF.agent.manage_agent_boot:
+                # prepare_ramdisk will set the boot device
+                deploy_utils.prepare_agent_boot(task)
 
         return states.RESCUEWAIT
 
@@ -1171,22 +1282,20 @@ class AgentRescue(base.RescueInterface):
         :raises: any boot interface's prepare_instance exceptions.
         :returns: Returns states.ACTIVE
         """
-        manager_utils.node_power_action(task, states.POWER_OFF)
+        with driver_utils.power_off_and_on(task):
+            # NOTE(TheJulia): Revealing that the power is off at any time can
+            # cause external power sync to decide that the node must be off.
+            # This may result in a post-rescued instance being turned off
+            # unexpectedly after unrescue.
+            # TODO(TheJulia): Once we have power/state callbacks to nova,
+            # the reset of the power_state can be removed.
+            task.node.power_state = states.POWER_ON
+            task.node.save()
 
-        # NOTE(TheJulia): Revealing that the power is off at any time can
-        # cause external power sync to decide that the node must be off.
-        # This may result in a post-rescued instance being turned off
-        # unexpectedly after unrescue.
-        # TODO(TheJulia): Once we have power/state callbacks to nova,
-        # the reset of the power_state can be removed.
-        task.node.power_state = states.POWER_ON
-        task.node.save()
-
-        self.clean_up(task)
-        with manager_utils.power_state_for_network_configuration(task):
-            task.driver.network.configure_tenant_networks(task)
-        task.driver.boot.prepare_instance(task)
-        manager_utils.node_power_action(task, states.POWER_ON)
+            self.clean_up(task)
+            with manager_utils.power_state_for_network_configuration(task):
+                task.driver.network.configure_tenant_networks(task)
+            task.driver.boot.prepare_instance(task)
 
         return states.ACTIVE
 
