@@ -16,16 +16,14 @@
 
 import os
 import re
-import time
 
-from ironic_lib import metrics_utils
-from ironic_lib import utils as il_utils
 from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import fileutils
 from oslo_utils import strutils
 
 from ironic.common import async_steps
+from ironic.common import checksum_utils
 from ironic.common import context
 from ironic.common import exception
 from ironic.common import faults
@@ -34,6 +32,7 @@ from ironic.common.i18n import _
 from ironic.common import image_service
 from ironic.common import images
 from ironic.common import keystone
+from ironic.common import metrics_utils
 from ironic.common import states
 from ironic.common import utils
 from ironic.conductor import utils as manager_utils
@@ -60,6 +59,7 @@ RESCUE_LIKE_STATES = (states.RESCUING, states.RESCUEWAIT, states.RESCUEFAIL,
 
 DISK_LAYOUT_PARAMS = ('root_gb', 'swap_mb', 'ephemeral_gb')
 
+
 # All functions are called from deploy() directly or indirectly.
 # They are split for stub-out.
 
@@ -206,7 +206,9 @@ def check_for_missing_params(info_dict, error_msg, param_prefix=''):
 
 
 def fetch_images(ctx, cache, images_info, force_raw=True,
-                 expected_format=None):
+                 expected_format=None, expected_checksum=None,
+                 expected_checksum_algo=None,
+                 image_auth_data=None):
     """Check for available disk space and fetch images using ImageCache.
 
     :param ctx: context
@@ -215,6 +217,10 @@ def fetch_images(ctx, cache, images_info, force_raw=True,
     :param force_raw: boolean value, whether to convert the image to raw
                       format
     :param expected_format: The expected format of the image.
+    :param expected_checksum: The expected image checksum, to be used if we
+           need to convert the image to raw prior to deploying.
+    :param expected_checksum_algo: The checksum algo in use, if separately
+           set.
     :raises: InstanceDeployFailure if unable to find enough disk space
     :raises: InvalidImage if the supplied image metadata or contents are
              deemed to be invalid, unsafe, or not matching the expectations
@@ -222,7 +228,8 @@ def fetch_images(ctx, cache, images_info, force_raw=True,
     """
 
     try:
-        image_cache.clean_up_caches(ctx, cache.master_dir, images_info)
+        image_cache.clean_up_caches(ctx, cache.master_dir, images_info,
+                                    image_auth_data)
     except exception.InsufficientDiskSpace as e:
         raise exception.InstanceDeployFailure(reason=e)
 
@@ -233,9 +240,13 @@ def fetch_images(ctx, cache, images_info, force_raw=True,
     image_list = []
     for href, path in images_info:
         # NOTE(TheJulia): Href in this case can be an image UUID or a URL.
-        image_format = cache.fetch_image(href, path, ctx=ctx,
-                                         force_raw=force_raw,
-                                         expected_format=expected_format)
+        image_format = cache.fetch_image(
+            href, path, ctx=ctx,
+            force_raw=force_raw,
+            expected_format=expected_format,
+            expected_checksum=expected_checksum,
+            expected_checksum_algo=expected_checksum_algo,
+            image_auth_data=image_auth_data)
         image_list.append((href, path, image_format))
     return image_list
 
@@ -763,7 +774,7 @@ def tear_down_inband_cleaning(task, manage_boot=True):
     node = task.node
     cleaning_failure = (node.fault == faults.CLEAN_FAILURE)
 
-    if not (fast_track or cleaning_failure):
+    if not (fast_track or cleaning_failure or node.disable_power_off):
         manager_utils.node_power_action(task, states.POWER_OFF)
 
     if manage_boot:
@@ -772,8 +783,11 @@ def tear_down_inband_cleaning(task, manage_boot=True):
     power_state_to_restore = manager_utils.power_on_node_if_needed(task)
     task.driver.network.remove_cleaning_network(task)
     if not (fast_track or cleaning_failure):
-        manager_utils.restore_power_state_if_needed(
-            task, power_state_to_restore)
+        if node.disable_power_off:
+            manager_utils.node_power_action(task, states.REBOOT)
+        else:
+            manager_utils.restore_power_state_if_needed(
+                task, power_state_to_restore)
 
 
 def prepare_inband_service(task):
@@ -788,24 +802,23 @@ def prepare_inband_service(task):
     :raises: any boot interface's prepare_ramdisk exceptions.
     :returns: Returns states.SERVICEWAIT
     """
-    manager_utils.node_power_action(task, states.POWER_OFF)
-    # NOTE(TheJulia): Revealing that the power is off at any time can
-    # cause external power sync to decide that the node must be off.
-    # This may result in a post-rescued instance being turned off
-    # unexpectedly after rescue has started.
-    # TODO(TheJulia): Once we have power/state callbacks to nova,
-    # the reset of the power_state can be removed.
-    task.node.power_state = states.POWER_ON
-    task.node.save()
+    with driver_utils.power_off_and_on(task):
+        # NOTE(TheJulia): Revealing that the power is off at any time can
+        # cause external power sync to decide that the node must be off.
+        # This may result in a post-rescued instance being turned off
+        # unexpectedly after rescue has started.
+        # TODO(TheJulia): Once we have power/state callbacks to nova,
+        # the reset of the power_state can be removed.
+        task.node.power_state = states.POWER_ON
+        task.node.save()
 
-    task.driver.boot.clean_up_instance(task)
-    with manager_utils.power_state_for_network_configuration(task):
-        task.driver.network.unconfigure_tenant_networks(task)
-        task.driver.network.add_servicing_network(task)
-    if CONF.agent.manage_agent_boot:
-        # prepare_ramdisk will set the boot device
-        prepare_agent_boot(task)
-    manager_utils.node_power_action(task, states.POWER_ON)
+        task.driver.boot.clean_up_instance(task)
+        with manager_utils.power_state_for_network_configuration(task):
+            task.driver.network.unconfigure_tenant_networks(task)
+            task.driver.network.add_servicing_network(task)
+        if CONF.agent.manage_agent_boot:
+            # prepare_ramdisk will set the boot device
+            prepare_agent_boot(task)
 
     return states.SERVICEWAIT
 
@@ -829,7 +842,7 @@ def tear_down_inband_service(task):
     node = task.node
     service_failure = (node.fault == faults.SERVICE_FAILURE)
 
-    if not service_failure:
+    if not service_failure and not node.disable_power_off:
         manager_utils.node_power_action(task, states.POWER_OFF)
 
     task.driver.boot.clean_up_ramdisk(task)
@@ -842,7 +855,9 @@ def tear_down_inband_service(task):
         task.driver.boot.prepare_instance(task)
         # prepare_instance does not power on the node, the deploy interface is
         # normally responsible for that.
-        manager_utils.node_power_action(task, states.POWER_ON)
+        next_state = (states.REBOOT if task.node.disable_power_off
+                      else states.POWER_ON)
+        manager_utils.node_power_action(task, next_state)
 
 
 def get_image_instance_info(node):
@@ -1072,7 +1087,8 @@ class InstanceImageCache(image_cache.ImageCache):
 
 
 @METRICS.timer('cache_instance_image')
-def cache_instance_image(ctx, node, force_raw=None, expected_format=None):
+def cache_instance_image(ctx, node, force_raw=None, expected_format=None,
+                         expected_checksum=None, expected_checksum_algo=None):
     """Fetch the instance's image from Glance
 
     This method pulls the disk image and writes them to the appropriate
@@ -1082,6 +1098,10 @@ def cache_instance_image(ctx, node, force_raw=None, expected_format=None):
     :param node: an ironic node object
     :param force_raw: whether convert image to raw format
     :param expected_format: The expected format of the disk image contents.
+    :param expected_checksum: The expected image checksum, to be used if we
+           need to convert the image to raw prior to deploying.
+    :param expected_checksum_algo: The checksum algo in use, if separately
+           set.
     :returns: a tuple containing the uuid of the image and the path in
         the filesystem where image is cached.
     :raises: InvalidImage if the requested image is invalid and cannot be
@@ -1095,13 +1115,22 @@ def cache_instance_image(ctx, node, force_raw=None, expected_format=None):
     i_info = parse_instance_info(node)
     fileutils.ensure_tree(_get_image_dir_path(node.uuid))
     image_path = _get_image_file_path(node.uuid)
-    uuid = i_info['image_source']
+
+    if 'image_source' in node.driver_internal_info:
+        uuid = node.driver_internal_info.get('image_source')
+    else:
+        uuid = i_info['image_source']
+
+    img_auth = image_service.get_image_service_auth_override(node)
 
     LOG.debug("Fetching image %(image)s for node %(uuid)s",
               {'image': uuid, 'uuid': node.uuid})
 
     image_list = fetch_images(ctx, InstanceImageCache(), [(uuid, image_path)],
-                              force_raw, expected_format=expected_format)
+                              force_raw, expected_format=expected_format,
+                              expected_checksum=expected_checksum,
+                              expected_checksum_algo=expected_checksum_algo,
+                              image_auth_data=img_auth)
     return (uuid, image_path, image_list[0][2])
 
 
@@ -1111,7 +1140,7 @@ def destroy_images(node_uuid):
 
     :param node_uuid: the uuid of the ironic node.
     """
-    il_utils.unlink_without_raise(_get_image_file_path(node_uuid))
+    utils.unlink_without_raise(_get_image_file_path(node_uuid))
     utils.rmtree_without_raise(_get_image_dir_path(node_uuid))
     InstanceImageCache().clean_up()
 
@@ -1119,22 +1148,16 @@ def destroy_images(node_uuid):
 @METRICS.timer('compute_image_checksum')
 def compute_image_checksum(image_path, algorithm='md5'):
     """Compute checksum by given image path and algorithm."""
-    time_start = time.time()
-    LOG.debug('Start computing %(algo)s checksum for image %(image)s.',
-              {'algo': algorithm, 'image': image_path})
-    checksum = fileutils.compute_file_checksum(image_path,
-                                               algorithm=algorithm)
-    time_elapsed = time.time() - time_start
-    LOG.debug('Computed %(algo)s checksum for image %(image)s in '
-              '%(delta).2f seconds, checksum value: %(checksum)s.',
-              {'algo': algorithm, 'image': image_path, 'delta': time_elapsed,
-               'checksum': checksum})
-    return checksum
+    # NOTE(TheJulia): This likely wouldn't be removed, but if we do
+    # significant refactoring we could likely just change everything
+    # over to the images common code, if we don't need the metrics
+    # data anymore.
+    return checksum_utils.compute_image_checksum(image_path, algorithm)
 
 
 def remove_http_instance_symlink(node_uuid):
     symlink_path = _get_http_image_symlink_file_path(node_uuid)
-    il_utils.unlink_without_raise(symlink_path)
+    utils.unlink_without_raise(symlink_path)
 
 
 def destroy_http_instance_images(node):
@@ -1181,7 +1204,13 @@ def _validate_image_url(node, url, secret=False, inspect_image=None,
         # is NOT raised. In other words, that the endpoint does not
         # return a 200. If we're fed a folder list, this will still
         # work, which is a good and bad thing at the same time. :/
-        image_service.HttpImageService().validate_href(url, secret)
+        if image_service.is_container_registry_url(url):
+            oci = image_service.OciImageService()
+            image_auth = image_service.get_image_service_auth_override(node)
+            oci.set_image_auth(url, image_auth)
+            oci.validate_href(url, secret)
+        else:
+            image_service.HttpImageService().validate_href(url, secret)
     except exception.ImageRefValidationFailed as e:
         with excutils.save_and_reraise_exception():
             LOG.error("The specified URL is not a valid HTTP(S) URL or is "
@@ -1205,6 +1234,11 @@ def _validate_image_url(node, url, secret=False, inspect_image=None,
         # let it run the image validation checking as it's normal course
         # of action, and save what it tells us the image format is.
         # if there *was* a mismatch, it will raise the error.
+
+        # NOTE(TheJulia): We don't need to supply the checksum here, because
+        # we are not converting the image. The net result is the deploy
+        # interface or remote agent has the responsibility to checksum the
+        # image.
         _, image_path, img_format = cache_instance_image(
             ctx,
             node,
@@ -1212,24 +1246,38 @@ def _validate_image_url(node, url, secret=False, inspect_image=None,
             expected_format=expected_format)
         # NOTE(TheJulia): We explicitly delete this file because it has no use
         # in the cache after this point.
-        il_utils.unlink_without_raise(image_path)
+        utils.unlink_without_raise(image_path)
         image_info['disk_format'] = img_format
     return image_info
 
 
 def _cache_and_convert_image(task, instance_info, image_info=None):
-    """Cache an image locally and convert it to RAW if needed."""
+    """Cache an image locally and convert it to RAW if needed.
+
+    :param task: The Taskmanager object related to this action.
+    :param instance_info: The instance_info field being used in
+                          association with this method call.
+    :param image_info: The supplied image_info from Glance.
+    """
     # Ironic cache and serve images from httpboot server
     force_raw = direct_deploy_should_convert_raw_image(task.node)
+    if 'image_source' in task.node.driver_internal_info:
+        image_source = task.node.driver_internal_info.get('image_source')
+    else:
+        image_source = task.node.instance_info.get('image_source')
 
     if image_info is None:
         initial_format = instance_info.get('image_disk_format')
     else:
         initial_format = image_info.get('disk_format')
+    checksum, checksum_algo = checksum_utils.get_checksum_and_algo(
+        instance_info)
     _, image_path, img_format = cache_instance_image(
         task.context, task.node,
         force_raw=force_raw,
-        expected_format=initial_format)
+        expected_format=initial_format,
+        expected_checksum=checksum,
+        expected_checksum_algo=checksum_algo)
     if force_raw or image_info is None:
         if force_raw:
             instance_info['image_disk_format'] = 'raw'
@@ -1238,7 +1286,7 @@ def _cache_and_convert_image(task, instance_info, image_info=None):
                       '%(image)s for node %(node)s',
                       {'image': image_path, 'node': task.node.uuid})
             instance_info['image_disk_format'] = \
-                images.get_source_format(instance_info['image_source'],
+                images.get_source_format(image_source,
                                          image_path)
 
         # Standard behavior is for image_checksum to be MD5,
@@ -1265,7 +1313,8 @@ def _cache_and_convert_image(task, instance_info, image_info=None):
                       '%(node)s due to image conversion',
                       {'image': image_path, 'node': task.node.uuid})
             instance_info['image_checksum'] = None
-            hash_value = compute_image_checksum(image_path, os_hash_algo)
+            hash_value = checksum_utils.compute_image_checksum(image_path,
+                                                               os_hash_algo)
         else:
             instance_info['image_checksum'] = old_checksum
 
@@ -1338,9 +1387,16 @@ def build_instance_info_for_deploy(task):
     """
     node = task.node
     instance_info = node.instance_info
+    di_info = node.driver_internal_info
     iwdi = node.driver_internal_info.get('is_whole_disk_image')
     image_source = instance_info['image_source']
 
+    # Remove the saved image_source in case it exists in driver_internal_info
+    di_info.pop('image_source', None)
+
+    # Save out driver_internal_info to prevent race conditions.
+    node.driver_internal_info = di_info
+
     # Flag if we know the source is a path, used for Anaconda
     # deploy interface where you can just tell anaconda to
     # consume artifacts from a path. In this case, we are not
@@ -1358,11 +1414,39 @@ def build_instance_info_for_deploy(task):
     # and gets replaced at various points in this sequence.
     instance_info['image_url'] = None
 
+    # This flag exists to lockout the overall continued flow of
+    # file validation if glance is in use. This is because glance
+    # can have have objects stored in Swift and those objects can
+    # be directly referenced by a separate swift client. Which means,
+    # additional information then needs to be gathered and exchanged
+    # which is a separate process from just a remote http file.
+    is_glance_image = False
+
+    # TODO(TheJulia): We should likely look at splitting this method
+    # into everal distinct helpers. First, glance, then OCI, then
+    # general file activities like download/cache or verify a remote
+    # URL.
+
+    # Remote image services/repositories are a little different, they house
+    # extra data we need to collect data from to streamline the process.
     if service_utils.is_glance_image(image_source):
+        # We know the image source is likely rooted from a glance record,
+        # so we don't need to do other checks unrelated to non-glance flows.
+        is_glance_image = True
+
+        # TODO(TheJulia): At some point, break all of the glance check/set
+        # work into a helper method to be called so we minimize the amount
+        # of glance specific code in this overall multi-image-service flow
+        # for future maintainer sanity.
         glance = image_service.GlanceImageService(context=task.context)
         image_info = glance.show(image_source)
         LOG.debug('Got image info: %(info)s for node %(node)s.',
                   {'info': image_info, 'node': node.uuid})
+        # Values are explicitly set into the instance info field
+        # so IPA have the values available.
+        instance_info['image_checksum'] = image_info['checksum']
+        instance_info['image_os_hash_algo'] = image_info['os_hash_algo']
+        instance_info['image_os_hash_value'] = image_info['os_hash_value']
         if image_download_source == 'swift':
             # In this case, we are getting a file *from* swift for a glance
             # image which is backed by swift. IPA downloads the file directly
@@ -1376,14 +1460,9 @@ def build_instance_info_for_deploy(task):
             validate_results = _validate_image_url(
                 node, swift_temp_url, secret=True,
                 expected_format=image_format)
-            # Values are explicitly set into the instance info field
-            # so IPA have the values available.
             instance_info['image_url'] = swift_temp_url
-            instance_info['image_checksum'] = image_info['checksum']
             instance_info['image_disk_format'] = \
                 validate_results.get('disk_format', image_format)
-            instance_info['image_os_hash_algo'] = image_info['os_hash_algo']
-            instance_info['image_os_hash_value'] = image_info['os_hash_value']
         else:
             # In this case, we're directly downloading the glance image and
             # hosting it locally for retrieval by the IPA.
@@ -1399,84 +1478,128 @@ def build_instance_info_for_deploy(task):
         if not iwdi and boot_option != 'local':
             instance_info['kernel'] = image_info['properties']['kernel_id']
             instance_info['ramdisk'] = image_info['properties']['ramdisk_id']
-    elif (image_source.startswith('file://')
-          or image_download_source == 'local'):
-        # In this case, we're explicitly downloading (or copying a file)
-        # hosted locally so IPA can download it directly from Ironic.
+    elif image_service.is_container_registry_url(image_source):
+        # Is an oci image, we need to figure out the particulars...
+        # but we *don't* need to also handle special casing with Swift.
+        # We will setup things so _cache_and_convert_image can do the needful
+        # Or just validate the remote url data.
+        oci = image_service.OciImageService()
+        image_auth = image_service.get_image_service_auth_override(task.node)
+        oci.set_image_auth(image_source, image_auth)
+        # Ask the image service method to identify and gather information
+        # about the image. This is different from a specific manifest supplied
+        # upfront.
+        image_info = oci.identify_specific_image(
+            image_source, image_download_source,
+            node.properties.get('cpu_arch')
+        )
+        if (image_info.get('image_disk_format') == 'unknown'
+            and instance_info.get('image_disk_format') == 'raw'):
+            # Ironic, internally, resets image_disk_format for IPA, and
+            # we're in a case where we've been given a specific URL, which
+            # might not be raw. There is no way to know what is actually
+            # correct, so we'll pop the value out completely, and let
+            # auto-detection run it's course, so rebuilds or redeploy
+            # attempts are an available option.
+            image_info.pop('image_disk_format')
+            instance_info.pop('image_disk_format')
+        instance_info.update(image_info)
+        # Save what we are using for discoverability by the user, and
+        # save an override image_source to driver_internal_info for
+        # other methods to rely upon as the authoritative source.
+        image_source = instance_info.get('oci_image_manifest_url')
+        # This preserves an override for _cache_and_convert_image
+        # so it knows where to actually retrieve data from without
+        # us overriding image_source saved by the user, so rebuilds
+        # will work as expected.
+        di_info['image_source'] = image_source
+        node.driver_internal_info = di_info
+    if not is_glance_image:
+        if (image_source.startswith('file://')
+                or image_download_source == 'local'):
+            # In this case, we're explicitly downloading (or copying a file)
+            # hosted locally so IPA can download it directly from Ironic.
 
-        # NOTE(TheJulia): Intentionally only supporting file:/// as image
-        # based deploy source since we don't want to, nor should we be in
-        # in the business of copying large numbers of files as it is a
-        # huge performance impact.
+            # NOTE(TheJulia): Intentionally only supporting file:/// as image
+            # based deploy source since we don't want to, nor should we be in
+            # in the business of copying large numbers of files as it is a
+            # huge performance impact.
 
-        _cache_and_convert_image(task, instance_info)
-    else:
-        # This is the "all other cases" logic for aspects like the user
-        # has supplied us a direct URL to reference. In cases like the
-        # anaconda deployment interface where we might just have a path
-        # and not a file, or where a user may be supplying a full URL to
-        # a remotely hosted image, we at a minimum need to check if the url
-        # is valid, and address any redirects upfront.
-        try:
-            # NOTE(TheJulia): In the case we're here, we not doing an
-            # integrated image based deploy, but we may also be doing
-            # a path based anaconda base deploy, in which case we have
-            # no backing image, but we need to check for a URL
-            # redirection. So, if the source is a path (i.e. isap),
-            # we don't need to inspect the image as there is no image
-            # in the case for the deployment to drive.
-            validated_results = {}
-            if isap:
-                # This is if the source is a path url, such as one used by
-                # anaconda templates to to rely upon bootstrapping defaults.
-                _validate_image_url(node, image_source, inspect_image=False)
-            else:
-                # When not isap, we can just let _validate_image_url make a
-                # the required decision on if contents need to be sampled,
-                # or not. We try to pass the image_disk_format which may be
-                # declared by the user, and if not we set expected_format to
-                # None.
-                validate_results = _validate_image_url(
-                    node,
-                    image_source,
-                    expected_format=instance_info.get('image_disk_format',
-                                                      None))
-            # image_url is internal, and used by IPA and some boot templates.
-            # in most cases, it needs to come from image_source explicitly.
-            if 'disk_format' in validated_results:
-                # Ensure IPA has the value available, so write what we detect,
-                # if anything. This is also an item which might be needful
-                # with ansible deploy interface, when used in standalone mode.
-                instance_info['image_disk_format'] = \
-                    validate_results.get('disk_format')
-            instance_info['image_url'] = image_source
-        except exception.ImageRefIsARedirect as e:
-            # At this point, we've got a redirect response from the webserver,
-            # and we're going to try to handle it as a single redirect action,
-            # as requests, by default, only lets a single redirect to occur.
-            # This is likely a URL pathing fix, like a trailing / on a path,
-            # or move to HTTPS from a user supplied HTTP url.
-            if e.redirect_url:
-                # Since we've got a redirect, we need to carry the rest of the
-                # request logic as well, which includes recording a disk
-                # format, if applicable.
-                instance_info['image_url'] = e.redirect_url
-                # We need to save the image_source back out so it caches
-                instance_info['image_source'] = e.redirect_url
-                task.node.instance_info = instance_info
-                if not isap:
-                    # The redirect doesn't relate to a path being used, so
-                    # the target is a filename, likely cause is webserver
-                    # telling the client to use HTTPS.
-                    validated_results = _validate_image_url(
-                        node, e.redirect_url,
-                        expected_format=instance_info.get('image_disk_format',
-                                                          None))
-                    if 'disk_format' in validated_results:
-                        instance_info['image_disk_format'] = \
-                            validated_results.get('disk_format')
-            else:
-                raise
+            _cache_and_convert_image(task, instance_info)
+        else:
+            # This is the "all other cases" logic for aspects like the user
+            # has supplied us a direct URL to reference. In cases like the
+            # anaconda deployment interface where we might just have a path
+            # and not a file, or where a user may be supplying a full URL to
+            # a remotely hosted image, we at a minimum need to check if the
+            # url is valid, and address any redirects upfront.
+            try:
+                # NOTE(TheJulia): In the case we're here, we not doing an
+                # integrated image based deploy, but we may also be doing
+                # a path based anaconda base deploy, in which case we have
+                # no backing image, but we need to check for a URL
+                # redirection. So, if the source is a path (i.e. isap),
+                # we don't need to inspect the image as there is no image
+                # in the case for the deployment to drive.
+                validated_results = {}
+                if isap:
+                    # This is if the source is a path url, such as one used by
+                    # anaconda templates to to rely upon bootstrapping
+                    # defaults.
+                    _validate_image_url(node, image_source,
+                                        inspect_image=False)
+                else:
+                    # When not isap, we can just let _validate_image_url make
+                    # the required decision on if contents need to be sampled,
+                    # or not. We try to pass the image_disk_format which may
+                    # be declared by the user, and if not we set
+                    # expected_format to None.
+                    validate_results = _validate_image_url(
+                        node,
+                        image_source,
+                        expected_format=instance_info.get(
+                            'image_disk_format',
+                            None))
+                # image_url is internal, and used by IPA and some boot
+                # templates. In most cases, it needs to come from image_source
+                # explicitly.
+                if 'disk_format' in validated_results:
+                    # Ensure IPA has the value available, so write what we
+                    # detect, if anything. This is also an item which might be
+                    # needful with ansible deploy interface, when used in
+                    # standalone mode.
+                    instance_info['image_disk_format'] = \
+                        validate_results.get('disk_format')
+                if not instance_info.get('image_url'):
+                    instance_info['image_url'] = image_source
+            except exception.ImageRefIsARedirect as e:
+                # At this point, we've got a redirect response from the
+                # webserver, and we're going to try to handle it as a single
+                # redirect action, as requests, by default, only lets a single
+                # redirect to occur. This is likely a URL pathing fix, like a
+                # trailing / on a path,
+                # or move to HTTPS from a user supplied HTTP url.
+                if e.redirect_url:
+                    # Since we've got a redirect, we need to carry the rest of
+                    # the request logic as well, which includes recording a
+                    # disk format, if applicable.
+                    instance_info['image_url'] = e.redirect_url
+                    # We need to save the image_source back out so it caches
+                    instance_info['image_source'] = e.redirect_url
+                    task.node.instance_info = instance_info
+                    if not isap:
+                        # The redirect doesn't relate to a path being used, so
+                        # the target is a filename, likely cause is webserver
+                        # telling the client to use HTTPS.
+                        validated_results = _validate_image_url(
+                            node, e.redirect_url,
+                            expected_format=instance_info.get(
+                                'image_disk_format', None))
+                        if 'disk_format' in validated_results:
+                            instance_info['image_disk_format'] = \
+                                validated_results.get('disk_format')
+                else:
+                    raise
 
     if not isap:
         if not iwdi:
@@ -1632,7 +1755,8 @@ def reboot_to_finish_step(task, timeout=None):
     disable_ramdisk = task.node.driver_internal_info.get(
         'cleaning_disable_ramdisk')
     if not disable_ramdisk:
-        if manager_utils.is_fast_track(task):
+        if (manager_utils.is_fast_track(task)
+                and not task.node.disable_power_off):
             LOG.debug('Forcing power off on node %s for a clean reboot into '
                       'the agent image', task.node)
             manager_utils.node_power_action(task, states.POWER_OFF)
@@ -1674,7 +1798,7 @@ def get_root_device_for_deploy(node):
         source = 'instance_info'
 
     try:
-        return il_utils.parse_root_device_hints(hints)
+        return utils.parse_root_device_hints(hints)
     except ValueError as e:
         raise exception.InvalidParameterValue(
             _('Failed to validate the root device hints %(hints)s (from the '
