@@ -15,7 +15,6 @@
 DRAC RAID specific methods
 """
 
-from ironic_lib import metrics_utils
 from oslo_log import log as logging
 from oslo_utils import importutils
 import sushy
@@ -23,6 +22,7 @@ import tenacity
 
 from ironic.common import exception
 from ironic.common.i18n import _
+from ironic.common import metrics_utils
 from ironic.common import states
 from ironic.conductor import periodics
 from ironic.conductor import utils as manager_utils
