@@ -15,9 +15,11 @@
 
 from http import client as http_client
 import socket
+import typing
 import urllib
 
 from oslo_log import log as logging
+from oslo_utils import excutils
 from oslo_utils import netutils
 import stevedore
 
@@ -32,7 +34,7 @@ from ironic import objects
 from ironic.objects import node_inventory
 
 LOG = logging.getLogger(__name__)
-_HOOKS_MGR = None
+_HOOKS_MGR = {}
 _OBJECT_NAME_PREFIX = 'inspector_data'
 AUTO_DISCOVERED_FLAG = 'auto_discovered'
 
@@ -469,34 +471,38 @@ def missing_entrypoints_callback(names):
     raise RuntimeError(error % ', '.join(names))
 
 
-def _inspection_hooks_manager(*args):
+def _inspection_hooks_manager(intf: str,
+                              enabled_hooks: typing.List[str],
+                              *args):
     """Create a Stevedore extension manager for inspection hooks.
 
+    :param intf: Inspection interface
+    :param enabled_hooks: Hooks to be enabled for the inspection interface
     :param args: arguments to pass to the hooks constructor
     :returns: a Stevedore NamedExtensionManager
     """
     global _HOOKS_MGR
-    if _HOOKS_MGR is None:
-        enabled_hooks = [x.strip()
-                         for x in CONF.inspector.hooks.split(',')
-                         if x.strip()]
-        _HOOKS_MGR = stevedore.NamedExtensionManager(
+    if _HOOKS_MGR.get(intf) is None:
+        _HOOKS_MGR[intf] = stevedore.NamedExtensionManager(
             'ironic.inspection.hooks',
             names=enabled_hooks,
             invoke_on_load=True,
             invoke_args=args,
             on_missing_entrypoints_callback=missing_entrypoints_callback,
             name_order=True)
-    return _HOOKS_MGR
+    return _HOOKS_MGR[intf]
 
 
-def validate_inspection_hooks():
+def validate_inspection_hooks(intf: str, enabled_hooks: typing.List[str]):
     """Validate the enabled inspection hooks.
 
+    :param intf: Inspection interface
+    :param enabled_hooks: Hooks to be enabled for the inspection interface
     :raises: RuntimeError on missing or failed to load hooks
     :returns: the list of hooks that passed validation
     """
-    conf_hooks = [ext for ext in _inspection_hooks_manager()]
+    conf_hooks = [ext for ext in
+                  _inspection_hooks_manager(intf, enabled_hooks)]
     valid_hooks = []
     valid_hook_names = set()
     errors = []
@@ -519,3 +525,97 @@ def validate_inspection_hooks():
         raise exception.HardwareInspectionFailure(error=msg)
 
     return valid_hooks
+
+
+def run_inspection_hooks(task,
+                         inventory,
+                         plugin_data,
+                         hooks,
+                         on_error_plugin_data):
+    """Process data from the ramdisk using inspection hooks."""
+
+    try:
+        _run_preprocess_hooks(task, inventory, plugin_data, hooks)
+    except exception.HardwareInspectionFailure:
+        with excutils.save_and_reraise_exception():
+            if on_error_plugin_data:
+                on_error_plugin_data(plugin_data, task.node)
+
+    try:
+        _run_post_hooks(task, inventory, plugin_data, hooks)
+    except exception.HardwareInspectionFailure:
+        with excutils.save_and_reraise_exception():
+            if on_error_plugin_data:
+                on_error_plugin_data(plugin_data, task.node)
+    except Exception as exc:
+        LOG.exception('Unexpected exception while running inspection hooks for'
+                      ' node %(node)s', {'node': task.node.uuid})
+        msg = _('Unexpected exception %(exc_class)s during processing for '
+                'node: %(node)s. Error: %(error)s' %
+                {'exc_class': exc.__class__.__name__,
+                 'node': task.node.uuid,
+                 'error': exc})
+        if on_error_plugin_data:
+            on_error_plugin_data(plugin_data, task.node)
+        raise exception.HardwareInspectionFailure(error=msg)
+
+
+def _run_preprocess_hooks(task, inventory, plugin_data, hooks):
+    """Executes the preprocess() for each hook.
+
+    :param task: a TaskManager instance
+    :param inventory: Hardware inventory information. Must not by modified.
+    :param plugin_data: Plugin data information.
+    :param hooks: List of hooks to execute.
+    :returns: nothing.
+    :raises: HardwareInspectionFailure on hook error
+    """
+    failures = []
+
+    for hook in hooks:
+        LOG.debug('Running preprocess inspection hook: %(hook)s for node: '
+                  '%(node)s', {'hook': hook.name, 'node': task.node.uuid})
+
+        # NOTE(dtantsur): catch exceptions, so that we have changes to update
+        # node inspection status with after look up
+        try:
+            hook.obj.preprocess(task, inventory, plugin_data)
+        except exception.HardwareInspectionFailure as exc:
+            LOG.error('Preprocess hook: %(hook)s failed for node %(node)s '
+                      'with error: %(error)s', {'hook': hook.name,
+                                                'node': task.node.uuid,
+                                                'error': exc})
+            failures.append(_('Error in preprocess hook %(hook)s: %(error)s' %
+                              {'hook': hook.name, 'error': exc}))
+        except Exception as exc:
+            LOG.exception('Preprocess hook: %(hook)s failed for node %(node)s '
+                          'with error: %(error)s', {'hook': hook.name,
+                                                    'node': task.node.uuid,
+                                                    'error': exc})
+            failures.append(_('Unexpected exception %(exc_class)s during '
+                              'preprocess hook %(hook)s: %(error)s' %
+                              {'exc_class': exc.__class__.__name__,
+                               'hook': hook.name, 'error': exc}))
+
+    if failures:
+        msg = _('The following failures happened while running preprocess '
+                'hooks for node %(node)s:\n%(failures)s' %
+                {'node': task.node.uuid, 'failures': '\n'.join(failures)})
+        raise exception.HardwareInspectionFailure(error=msg)
+
+    # TODO(masghar): Store unprocessed inspection data
+
+
+def _run_post_hooks(task, inventory, plugin_data, hooks):
+    """Executes each supplied hook.
+
+    :param task: a TaskManager instance
+    :param inventory: Hardware inventory information. Must not by modified.
+    :param plugin_data: Plugin data information.
+    :param hooks: List of hooks to execute.
+    :returns: nothing.
+    """
+    for hook in hooks:
+        LOG.debug('Running inspection hook %(hook)s for node %(node)s',
+                  {'hook': hook.name, 'node': task.node.uuid})
+        hook.obj.__call__(task, inventory, plugin_data)
