@@ -20,6 +20,7 @@ from ironic.drivers.modules import noop_mgmt
 from ironic.drivers.modules.redfish import bios as redfish_bios
 from ironic.drivers.modules.redfish import boot as redfish_boot
 from ironic.drivers.modules.redfish import firmware as redfish_firmware
+from ironic.drivers.modules.redfish import graphical_console
 from ironic.drivers.modules.redfish import inspect as redfish_inspect
 from ironic.drivers.modules.redfish import management as redfish_mgmt
 from ironic.drivers.modules.redfish import power as redfish_power
@@ -73,3 +74,8 @@ class RedfishHardware(generic.GenericHardware):
     @property
     def supported_firmware_interfaces(self):
         return [redfish_firmware.RedfishFirmware, noop.NoFirmware]
+
+    @property
+    def supported_console_interfaces(self):
+        """List of supported console interfaces."""
+        return [graphical_console.RedfishGraphicalConsole, noop.NoConsole]
