@@ -12,6 +12,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import contextlib
 import os
 import tempfile
 
@@ -184,6 +185,27 @@ def add_node_capability(task, capability, value):
     node.save()
 
 
+def remove_node_capability(task, name):
+    """Remove 'capability' from node's 'capabilities' property.
+
+    If 'capability' is empty, do nothing.
+
+    :param task: Task object.
+    :param capability: Capability key.
+
+    """
+    node = task.node
+    properties = node.properties
+    capabilities = properties.get('capabilities', '').split(',')
+    if not capabilities:
+        return
+    updated_capabilities = [cap for cap in capabilities
+                            if not cap.startswith("%s:" % name)]
+    properties['capabilities'] = ','.join(updated_capabilities or [])
+    node.properties = properties
+    node.save()
+
+
 def ensure_next_boot_device(task, driver_info):
     """Ensure boot from correct device if persistent is True
 
@@ -501,4 +523,21 @@ def need_prepare_ramdisk(node):
                                     states.RESCUING,
                                     states.RESCUEWAIT,
                                     states.INSPECTING,
-                                    states.INSPECTWAIT)
+                                    states.INSPECTWAIT,
+                                    states.SERVICING,
+                                    states.SERVICEWAIT)
+
+
+@contextlib.contextmanager
+def power_off_and_on(task):
+    """A context manager that handles a reboot.
+
+    If disable_power_off is False, the node is powered off before yielding and
+    powered back on afterwards. Otherwise, one reboot is issued in the end.
+    """
+    if not task.node.disable_power_off:
+        utils.node_power_action(task, states.POWER_OFF)
+    yield
+    next_state = (states.REBOOT if task.node.disable_power_off
+                  else states.POWER_ON)
+    utils.node_power_action(task, next_state)
