@@ -23,10 +23,11 @@ from unittest import mock
 
 from oslo_concurrency import processutils
 from oslo_config import cfg
+from oslo_utils import fileutils
+from oslo_utils.imageutils import format_inspector as image_format_inspector
 
 from ironic.common import exception
 from ironic.common.glance_service import service_utils as glance_utils
-from ironic.common import image_format_inspector
 from ironic.common import image_service
 from ironic.common import images
 from ironic.common import qemu_img
@@ -41,9 +42,11 @@ class IronicImagesTestCase(base.TestCase):
     class FakeImgInfo(object):
         pass
 
+    @mock.patch.object(images, '_handle_zstd_compression', autospec=True)
     @mock.patch.object(image_service, 'get_image_service', autospec=True)
     @mock.patch.object(builtins, 'open', autospec=True)
-    def test_fetch_image_service(self, open_mock, image_service_mock):
+    def test_fetch_image_service(self, open_mock, image_service_mock,
+                                 mock_zstd):
         mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'file'
         open_mock.return_value = mock_file_handle
@@ -55,12 +58,16 @@ class IronicImagesTestCase(base.TestCase):
                                                    context='context')
         image_service_mock.return_value.download.assert_called_once_with(
             'image_href', 'file')
+        mock_zstd.assert_called_once_with('path')
 
+    @mock.patch.object(images, '_handle_zstd_compression', autospec=True)
     @mock.patch.object(image_service, 'get_image_service', autospec=True)
     @mock.patch.object(images, 'image_to_raw', autospec=True)
     @mock.patch.object(builtins, 'open', autospec=True)
     def test_fetch_image_service_force_raw(self, open_mock, image_to_raw_mock,
-                                           image_service_mock):
+                                           image_service_mock,
+                                           mock_zstd):
+        image_service_mock.return_value.transfer_verified_checksum = None
         mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'file'
         open_mock.return_value = mock_file_handle
@@ -72,6 +79,148 @@ class IronicImagesTestCase(base.TestCase):
             'image_href', 'file')
         image_to_raw_mock.assert_called_once_with(
             'image_href', 'path', 'path.part')
+        mock_zstd.assert_called_once_with('path')
+
+    @mock.patch.object(images, '_handle_zstd_compression', autospec=True)
+    @mock.patch.object(fileutils, 'compute_file_checksum',
+                       autospec=True)
+    @mock.patch.object(image_service, 'get_image_service', autospec=True)
+    @mock.patch.object(images, 'image_to_raw', autospec=True)
+    @mock.patch.object(builtins, 'open', autospec=True)
+    def test_fetch_image_service_force_raw_with_checksum(
+            self, open_mock, image_to_raw_mock,
+            image_service_mock, mock_checksum, mock_zstd):
+        image_service_mock.return_value.transfer_verified_checksum = None
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
+        mock_file_handle.__enter__.return_value = 'file'
+        open_mock.return_value = mock_file_handle
+        mock_checksum.return_value = 'f00'
+
+        images.fetch('context', 'image_href', 'path', force_raw=True,
+                     checksum='f00', checksum_algo='sha256')
+
+        mock_checksum.assert_called_once_with('path', algorithm='sha256')
+        open_mock.assert_called_once_with('path', 'wb')
+        image_service_mock.return_value.download.assert_called_once_with(
+            'image_href', 'file')
+        image_to_raw_mock.assert_called_once_with(
+            'image_href', 'path', 'path.part')
+        mock_zstd.assert_called_once_with('path')
+
+    @mock.patch.object(images, '_handle_zstd_compression', autospec=True)
+    @mock.patch.object(fileutils, 'compute_file_checksum',
+                       autospec=True)
+    @mock.patch.object(image_service, 'get_image_service', autospec=True)
+    @mock.patch.object(images, 'image_to_raw', autospec=True)
+    @mock.patch.object(builtins, 'open', autospec=True)
+    def test_fetch_image_service_with_checksum_mismatch(
+            self, open_mock, image_to_raw_mock,
+            image_service_mock, mock_checksum,
+            mock_zstd):
+        image_service_mock.return_value.transfer_verified_checksum = None
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
+        mock_file_handle.__enter__.return_value = 'file'
+        open_mock.return_value = mock_file_handle
+        mock_checksum.return_value = 'a00'
+
+        self.assertRaises(exception.ImageChecksumError,
+                          images.fetch, 'context', 'image_href',
+                          'path', force_raw=True,
+                          checksum='f00', checksum_algo='sha256')
+
+        mock_checksum.assert_called_once_with('path', algorithm='sha256')
+        open_mock.assert_called_once_with('path', 'wb')
+        image_service_mock.return_value.download.assert_called_once_with(
+            'image_href', 'file')
+        # If the checksum fails, then we don't attempt to convert the image.
+        image_to_raw_mock.assert_not_called()
+        mock_zstd.assert_not_called()
+
+    @mock.patch.object(images, '_handle_zstd_compression', autospec=True)
+    @mock.patch.object(fileutils, 'compute_file_checksum',
+                       autospec=True)
+    @mock.patch.object(image_service, 'get_image_service', autospec=True)
+    @mock.patch.object(images, 'image_to_raw', autospec=True)
+    @mock.patch.object(builtins, 'open', autospec=True)
+    def test_fetch_image_service_force_raw_no_checksum_algo(
+            self, open_mock, image_to_raw_mock,
+            image_service_mock, mock_checksum,
+            mock_zstd):
+        image_service_mock.return_value.transfer_verified_checksum = None
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
+        mock_file_handle.__enter__.return_value = 'file'
+        open_mock.return_value = mock_file_handle
+        mock_checksum.return_value = 'f00'
+
+        images.fetch('context', 'image_href', 'path', force_raw=True,
+                     checksum='f00')
+
+        mock_checksum.assert_called_once_with('path', algorithm='md5')
+        open_mock.assert_called_once_with('path', 'wb')
+        image_service_mock.return_value.download.assert_called_once_with(
+            'image_href', 'file')
+        image_to_raw_mock.assert_called_once_with(
+            'image_href', 'path', 'path.part')
+        mock_zstd.assert_called_once_with('path')
+
+    @mock.patch.object(images, '_handle_zstd_compression', autospec=True)
+    @mock.patch.object(fileutils, 'compute_file_checksum',
+                       autospec=True)
+    @mock.patch.object(image_service, 'get_image_service', autospec=True)
+    @mock.patch.object(images, 'image_to_raw', autospec=True)
+    @mock.patch.object(builtins, 'open', autospec=True)
+    def test_fetch_image_service_force_raw_combined_algo(
+            self, open_mock, image_to_raw_mock,
+            image_service_mock, mock_checksum,
+            mock_zstd):
+        image_service_mock.return_value.transfer_verified_checksum = None
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
+        mock_file_handle.__enter__.return_value = 'file'
+        open_mock.return_value = mock_file_handle
+        mock_checksum.return_value = 'f00'
+
+        images.fetch('context', 'image_href', 'path', force_raw=True,
+                     checksum='sha512:f00')
+
+        mock_checksum.assert_called_once_with('path', algorithm='sha512')
+        open_mock.assert_called_once_with('path', 'wb')
+        image_service_mock.return_value.download.assert_called_once_with(
+            'image_href', 'file')
+        image_to_raw_mock.assert_called_once_with(
+            'image_href', 'path', 'path.part')
+        mock_zstd.assert_called_once_with('path')
+
+    @mock.patch.object(images, '_handle_zstd_compression', autospec=True)
+    @mock.patch.object(fileutils, 'compute_file_checksum',
+                       autospec=True)
+    @mock.patch.object(image_service, 'get_image_service', autospec=True)
+    @mock.patch.object(images, 'image_to_raw', autospec=True)
+    @mock.patch.object(builtins, 'open', autospec=True)
+    def test_fetch_image_service_auth_data_checksum(
+            self, open_mock, image_to_raw_mock,
+            svc_mock, mock_checksum,
+            mock_zstd):
+        svc_mock.return_value.transfer_verified_checksum = 'f00'
+        svc_mock.return_value.is_auth_set_needed = True
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
+        mock_file_handle.__enter__.return_value = 'file'
+        open_mock.return_value = mock_file_handle
+        mock_checksum.return_value = 'f00'
+
+        images.fetch('context', 'image_href', 'path', force_raw=True,
+                     checksum='sha512:f00', image_auth_data='meow')
+        # In this case, the image service does the checksum so we know
+        # we don't need to do a checksum pass as part of the common image
+        # handling code path.
+        mock_checksum.assert_not_called()
+        open_mock.assert_called_once_with('path', 'wb')
+        svc_mock.return_value.download.assert_called_once_with(
+            'image_href', 'file')
+        image_to_raw_mock.assert_called_once_with(
+            'image_href', 'path', 'path.part')
+        svc_mock.return_value.set_image_auth.assert_called_once_with(
+            'image_href', 'meow')
+        mock_zstd.assert_called_once_with('path')
 
     @mock.patch.object(image_format_inspector, 'detect_file_format',
                        autospec=True)
@@ -79,7 +228,6 @@ class IronicImagesTestCase(base.TestCase):
         info = mock.MagicMock()
         # In the case the image looks okay, but it is not in our permitted
         # format list, we need to ensure we still fail appropriately.
-        info.safety_check.return_value = True
         info.__str__.return_value = 'vhd'
         detect_format_mock.return_value = info
 
@@ -94,7 +242,8 @@ class IronicImagesTestCase(base.TestCase):
     def test_image_to_raw_fails_safety_check(self, detect_format_mock):
         info = mock.MagicMock()
         info.__str__.return_value = 'qcow2'
-        info.safety_check.return_value = False
+        info.safety_check.side_effect = \
+            image_format_inspector.SafetyCheckFailed({"I'm a teapot": True})
         detect_format_mock.return_value = info
 
         e = self.assertRaises(exception.ImageUnacceptable, images.image_to_raw,
@@ -102,6 +251,8 @@ class IronicImagesTestCase(base.TestCase):
         info.safety_check.assert_called_once()
         detect_format_mock.assert_called_once_with('path_tmp')
         self.assertIn("The requested image is not valid for use.", str(e))
+        # Do not disclose the actual error message to evil hackers
+        self.assertNotIn("I'm a teapot", str(e))
 
     @mock.patch.object(os, 'rename', autospec=True)
     @mock.patch.object(os, 'unlink', autospec=True)
@@ -114,7 +265,6 @@ class IronicImagesTestCase(base.TestCase):
         info = mock.MagicMock()
         info.__str__.side_effect = iter(['qcow2', 'raw'])
         info.backing_file = None
-        info.saftey_check.return_value = True
         detect_format_mock.return_value = info
 
         def convert_side_effect(source, dest, out_format, source_format):
@@ -133,6 +283,67 @@ class IronicImagesTestCase(base.TestCase):
         unlink_mock.assert_called_once_with('path_tmp')
         rename_mock.assert_called_once_with('path.converted', 'path')
 
+    @mock.patch.object(os, 'rename', autospec=True)
+    @mock.patch.object(os, 'unlink', autospec=True)
+    @mock.patch.object(qemu_img, 'convert_image', autospec=True)
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    def test_image_to_gpt(self, detect_format_mock, convert_image_mock,
+                          unlink_mock, rename_mock):
+        CONF.set_override('force_raw_images', True)
+        info = mock.MagicMock()
+        info.__str__.side_effect = iter(['qcow2', 'gpt'])
+        info.backing_file = None
+        detect_format_mock.return_value = info
+
+        def convert_side_effect(source, dest, out_format, source_format):
+            info.file_format = 'gpt'
+        convert_image_mock.side_effect = convert_side_effect
+
+        images.image_to_raw('image_href', 'path', 'path_tmp')
+        info.safety_check.assert_called_once()
+        self.assertEqual(2, info.__str__.call_count)
+        detect_format_mock.assert_has_calls([
+            mock.call('path_tmp'),
+            mock.call('path.converted')])
+        convert_image_mock.assert_called_once_with('path_tmp',
+                                                   'path.converted', 'raw',
+                                                   source_format='qcow2')
+        unlink_mock.assert_called_once_with('path_tmp')
+        rename_mock.assert_called_once_with('path.converted', 'path')
+
+    @mock.patch.object(os, 'rename', autospec=True)
+    @mock.patch.object(os, 'unlink', autospec=True)
+    @mock.patch.object(qemu_img, 'convert_image', autospec=True)
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    def test_image_to_gpt_backward_compatibility(self, detect_format_mock,
+                                                 convert_image_mock,
+                                                 unlink_mock, rename_mock):
+        CONF.set_override('force_raw_images', True)
+        CONF.set_override('permitted_image_formats', 'raw,qcow2',
+                          group='conductor')
+        info = mock.MagicMock()
+        info.__str__.side_effect = iter(['qcow2', 'gpt'])
+        info.backing_file = None
+        detect_format_mock.return_value = info
+
+        def convert_side_effect(source, dest, out_format, source_format):
+            info.file_format = 'gpt'
+        convert_image_mock.side_effect = convert_side_effect
+
+        images.image_to_raw('image_href', 'path', 'path_tmp')
+        info.safety_check.assert_called_once()
+        self.assertEqual(2, info.__str__.call_count)
+        detect_format_mock.assert_has_calls([
+            mock.call('path_tmp'),
+            mock.call('path.converted')])
+        convert_image_mock.assert_called_once_with('path_tmp',
+                                                   'path.converted', 'raw',
+                                                   source_format='qcow2')
+        unlink_mock.assert_called_once_with('path_tmp')
+        rename_mock.assert_called_once_with('path.converted', 'path')
+
     @mock.patch.object(os, 'rename', autospec=True)
     @mock.patch.object(os, 'unlink', autospec=True)
     @mock.patch.object(qemu_img, 'convert_image', autospec=True)
@@ -147,7 +358,6 @@ class IronicImagesTestCase(base.TestCase):
         info = mock.MagicMock()
         info.__str__.side_effect = iter(['vmdk', 'raw'])
         info.backing_file = None
-        info.saftey_check.return_value = None
         detect_format_mock.return_value = info
 
         def convert_side_effect(source, dest, out_format, source_format):
@@ -179,7 +389,6 @@ class IronicImagesTestCase(base.TestCase):
         info = mock.MagicMock()
         info.__str__.return_value = 'vmdk'
         info.backing_file = None
-        info.saftey_check.return_value = None
         detect_format_mock.return_value = info
 
         self.assertRaises(exception.ImageConvertFailed,
@@ -214,7 +423,6 @@ class IronicImagesTestCase(base.TestCase):
                                                    source_format='qcow2')
         unlink_mock.assert_called_once_with('path_tmp')
         info.safety_check.assert_called_once()
-        info.safety_check.assert_called_once()
         self.assertEqual(2, info.__str__.call_count)
         detect_format_mock.assert_has_calls([
             mock.call('path_tmp'),
@@ -233,6 +441,21 @@ class IronicImagesTestCase(base.TestCase):
 
         rename_mock.assert_called_once_with('path_tmp', 'path')
         info.safety_check.assert_called_once()
+        self.assertEqual(1, info.__str__.call_count)
+        detect_format_mock.assert_called_once_with('path_tmp')
+
+    @mock.patch.object(os, 'rename', autospec=True)
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    def test_image_to_raw_already_gpt_format(self, detect_format_mock,
+                                             rename_mock):
+        info = mock.MagicMock()
+        info.__str__.return_value = 'gpt'
+        detect_format_mock.return_value = info
+
+        images.image_to_raw('image_href', 'path', 'path_tmp')
+
+        rename_mock.assert_called_once_with('path_tmp', 'path')
         info.safety_check.assert_called_once()
         self.assertEqual(1, info.__str__.call_count)
         detect_format_mock.assert_called_once_with('path_tmp')
@@ -269,16 +492,18 @@ class IronicImagesTestCase(base.TestCase):
     @mock.patch.object(images, 'image_show', autospec=True)
     def test_download_size(self, show_mock):
         show_mock.return_value = {'size': 123456}
-        size = images.download_size('context', 'image_href', 'image_service')
+        size = images.download_size('context', 'image_href', 'image_service',
+                                    image_auth_data='meow')
         self.assertEqual(123456, size)
         show_mock.assert_called_once_with('context', 'image_href',
-                                          'image_service')
+                                          image_service='image_service',
+                                          image_auth_data='meow')
 
     @mock.patch.object(image_format_inspector, 'detect_file_format',
                        autospec=True)
     def test_converted_size_estimate_default(self, image_info_mock):
         info = self.FakeImgInfo()
-        info.disk_size = 2
+        info.actual_size = 2
         info.virtual_size = 10 ** 10
         image_info_mock.return_value = info
         size = images.converted_size('path', estimate=True)
@@ -290,7 +515,7 @@ class IronicImagesTestCase(base.TestCase):
     def test_converted_size_estimate_custom(self, image_info_mock):
         CONF.set_override('raw_image_growth_factor', 3)
         info = self.FakeImgInfo()
-        info.disk_size = 2
+        info.actual_size = 2
         info.virtual_size = 10 ** 10
         image_info_mock.return_value = info
         size = images.converted_size('path', estimate=True)
@@ -302,7 +527,7 @@ class IronicImagesTestCase(base.TestCase):
     def test_converted_size_estimate_raw_smaller(self, image_info_mock):
         CONF.set_override('raw_image_growth_factor', 3)
         info = self.FakeImgInfo()
-        info.disk_size = 2
+        info.actual_size = 2
         info.virtual_size = 5
         image_info_mock.return_value = info
         size = images.converted_size('path', estimate=True)
@@ -371,6 +596,25 @@ class IronicImagesTestCase(base.TestCase):
         mock_igi.assert_called_once_with(image_source)
         mock_gip.assert_called_once_with('context', image_source)
 
+    @mock.patch.object(images, 'get_image_properties', autospec=True)
+    @mock.patch.object(image_service, 'is_container_registry_url',
+                       autospec=True)
+    @mock.patch.object(glance_utils, 'is_glance_image', autospec=True)
+    def test_is_whole_disk_image_whole_disk_image_oci(self, mock_igi,
+                                                      mock_ioi,
+                                                      mock_gip):
+        mock_igi.return_value = False
+        mock_ioi.return_value = True
+        mock_gip.return_value = {}
+        instance_info = {'image_source': 'oci://image'}
+        image_source = instance_info['image_source']
+        is_whole_disk_image = images.is_whole_disk_image('context',
+                                                         instance_info)
+        self.assertTrue(is_whole_disk_image)
+        mock_igi.assert_called_once_with(image_source)
+        mock_ioi.assert_called_once_with(image_source)
+        mock_gip.assert_not_called()
+
     @mock.patch.object(images, 'get_image_properties', autospec=True)
     @mock.patch.object(glance_utils, 'is_glance_image', autospec=True)
     def test_is_whole_disk_image_partition_non_glance(self, mock_igi,
@@ -458,6 +702,32 @@ class IronicImagesTestCase(base.TestCase):
         validate_mock.side_effect = OSError
         self.assertIsNone(images.is_source_a_path('context', url))
 
+    @mock.patch.object(utils, 'execute', autospec=True)
+    @mock.patch.object(shutil, 'move', autospec=True)
+    @mock.patch.object(builtins, 'open', autospec=True)
+    def test__hanlde_zstd_compression(self, mock_open, mock_move,
+                                      mock_exec):
+        mock_file_handle = mock.Mock()
+        mock_file_handle.read.return_value = b"\x28\xb5\x2f\xfd"
+        mock_open.return_value.__enter__.open = mock_file_handle
+        images._handle_zstd_compression('path')
+        mock_move.assert_called_once_with('path', 'path.zstd')
+        mock_exec.assert_called_once_with('zstd', '-d', '--rm', 'path.zstd')
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    @mock.patch.object(shutil, 'move', autospec=True)
+    @mock.patch.object(builtins, 'open', autospec=True)
+    def test__hanlde_zstd_compression_disabled(self, mock_open, mock_move,
+                                               mock_exec):
+        mock_file_handle = mock.Mock()
+        mock_file_handle.read.return_value = b"\x28\xb5\x2f\xfd"
+        mock_open.return_value.__enter__.open = mock_file_handle
+        CONF.set_override('disable_zstandard_decompression', True,
+                          group='conductor')
+        images._handle_zstd_compression('path')
+        mock_move.assert_not_called()
+        mock_exec.assert_not_called()
+
 
 class FsImageTestCase(base.TestCase):
 
@@ -559,8 +829,8 @@ class FsImageTestCase(base.TestCase):
                         "set hidden_timeout_quiet=false\n"
                         "\n"
                         "menuentry \"boot_partition\" {\n"
-                        "linuxefi /vmlinuz key1=value1 key2 --\n"
-                        "initrdefi /initrd\n"
+                        "linux /vmlinuz key1=value1 key2 --\n"
+                        "initrd /initrd\n"
                         "}")
 
         cfg = images._generate_cfg(kernel_params,
