@@ -10,6 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import copy
 import os
 import tempfile
 from unittest import mock
@@ -18,49 +19,46 @@ import fixtures
 import oslo_messaging
 import webob
 
-from ironic.common import context as ir_ctx
 from ironic.common import exception
 from ironic.common.json_rpc import client
 from ironic.common.json_rpc import server
-from ironic import objects
-from ironic.objects import base as objects_base
-from ironic.tests import base as test_base
-from ironic.tests.unit.db import utils as db_utils
-from ironic.tests.unit.objects import utils as obj_utils
+from ironic.tests.base import TestCase
+
+
+class FakeContext(server.EmptyContext):
+
+    request_id = 'abcd'
 
 
 class FakeManager(object):
 
     def success(self, context, x, y=0):
-        assert isinstance(context, ir_ctx.RequestContext)
+        assert isinstance(context, FakeContext)
         assert context.user_name == 'admin'
         return x - y
 
-    def with_node(self, context, node):
-        assert isinstance(context, ir_ctx.RequestContext)
-        assert isinstance(node, objects.Node)
-        node.extra['answer'] = 42
-        return node
-
     def no_result(self, context):
-        assert isinstance(context, ir_ctx.RequestContext)
+        assert isinstance(context, FakeContext)
         return None
 
     def no_context(self):
         return 42
 
     def fail(self, context, message):
-        assert isinstance(context, ir_ctx.RequestContext)
+        assert isinstance(context, FakeContext)
         raise exception.IronicException(message)
 
-    @oslo_messaging.expected_exceptions(exception.Invalid)
+    @oslo_messaging.expected_exceptions(exception.BadRequest)
     def expected(self, context, message):
-        assert isinstance(context, ir_ctx.RequestContext)
-        raise exception.Invalid(message)
+        assert isinstance(context, FakeContext)
+        raise exception.BadRequest(message)
 
     def crash(self, context):
         raise RuntimeError('boom')
 
+    def copy(self, context, data):
+        return copy.deepcopy(data)
+
     def init_host(self, context):
         assert False, "This should not be exposed"
 
@@ -71,7 +69,16 @@ class FakeManager(object):
     value = 42
 
 
-class TestService(test_base.TestCase):
+class FakeSerializer:
+
+    def serialize_entity(self, context, entity):
+        return entity
+
+    def deserialize_entity(self, context, data):
+        return data
+
+
+class TestService(TestCase):
 
     def setUp(self):
         super(TestService, self).setUp()
@@ -79,8 +86,9 @@ class TestService(test_base.TestCase):
         self.server_mock = self.useFixture(fixtures.MockPatch(
             'oslo_service.wsgi.Server', autospec=True)).mock
 
-        self.serializer = objects_base.IronicObjectSerializer(is_server=True)
-        self.service = server.WSGIService(FakeManager(), self.serializer)
+        self.serializer = FakeSerializer()
+        self.service = server.WSGIService(FakeManager(), self.serializer,
+                                          FakeContext)
         self.app = self.service._application
         self.ctx = {'user_name': 'admin'}
 
@@ -130,7 +138,8 @@ class TestService(test_base.TestCase):
         self.config(auth_strategy='http_basic', group='json_rpc')
         # self.config(http_basic_username='myUser', group='json_rpc')
         # self.config(http_basic_password='myPassword', group='json_rpc')
-        self.service = server.WSGIService(FakeManager(), self.serializer)
+        self.service = server.WSGIService(FakeManager(), self.serializer,
+                                          FakeContext)
         self.app = self.server_mock.call_args[0][2]
 
     def test_http_basic_not_authenticated(self):
@@ -165,15 +174,6 @@ class TestService(test_base.TestCase):
         body = self._request('no_context')
         self._check(body, result=42)
 
-    def test_serialize_objects(self):
-        node = obj_utils.get_test_node(self.context)
-        node = self.serializer.serialize_entity(self.context, node)
-        body = self._request('with_node', {'context': self.ctx, 'node': node})
-        self.assertNotIn('error', body)
-        self.assertIsInstance(body['result'], dict)
-        node = self.serializer.deserialize_entity(self.context, body['result'])
-        self.assertEqual({'answer': 42}, node.extra)
-
     def test_non_json_body(self):
         for body in (b'', b'???', b"\xc3\x28"):
             request = webob.Request.blank("/", method='POST', body=body)
@@ -230,7 +230,7 @@ class TestService(test_base.TestCase):
                         'message': 'some error',
                         'code': 500,
                         'data': {
-                            'class': 'ironic_lib.exception.IronicException'
+                            'class': 'ironic.common.exception.IronicException'
                         }
                     })
 
@@ -266,7 +266,7 @@ class TestService(test_base.TestCase):
                         'code': -32601,
                     })
 
-    def test_no_blacklisted_methods(self):
+    def test_no_deny_methods(self):
         for name in ('__init__', '_private', 'init_host', 'value'):
             body = self._request(name, {'context': self.ctx})
             self._check(body,
@@ -287,40 +287,62 @@ class TestService(test_base.TestCase):
 
     def test_authenticated(self):
         self.config(auth_strategy='keystone', group='json_rpc')
-        self.service = server.WSGIService(FakeManager(), self.serializer)
+        self.service = server.WSGIService(FakeManager(), self.serializer,
+                                          FakeContext)
         self.app = self.server_mock.call_args[0][2]
         self._request('success', {'context': self.ctx, 'x': 42},
                       expected_error=401)
 
+    def test_authenticated_with_allowed_role(self):
+        self.config(auth_strategy='keystone', group='json_rpc')
+        self.config(allowed_roles=['allowed', 'ignored'], group='json_rpc')
+        self.service = server.WSGIService(FakeManager(), self.serializer,
+                                          FakeContext)
+        self.app = self.server_mock.call_args[0][2]
+        self._request('success', {'context': self.ctx, 'x': 42},
+                      expected_error=401,
+                      headers={'Content-Type': 'application/json',
+                               'X-Roles': 'allowed,denied'})
+
     def test_authenticated_no_admin_role(self):
         self.config(auth_strategy='keystone', group='json_rpc')
         self._request('success', {'context': self.ctx, 'x': 42},
                       expected_error=403)
 
+    def test_authenticated_no_allowed_role(self):
+        self.config(auth_strategy='keystone', group='json_rpc')
+        self.config(allowed_roles=['allowed', 'ignored'], group='json_rpc')
+        self._request('success', {'context': self.ctx, 'x': 42},
+                      expected_error=403,
+                      headers={'Content-Type': 'application/json',
+                               'X-Roles': 'denied,notallowed'})
+
     @mock.patch.object(server.LOG, 'debug', autospec=True)
     def test_mask_secrets(self, mock_log):
-        node = obj_utils.get_test_node(
-            self.context, driver_info=db_utils.get_test_ipmi_info())
-        node = self.serializer.serialize_entity(self.context, node)
-        body = self._request('with_node', {'context': self.ctx, 'node': node})
-        node = self.serializer.deserialize_entity(self.context, body['result'])
+        data = {'ipmi_username': 'admin', 'ipmi_password': 'secret'}
+        node = self.serializer.serialize_entity(self.ctx, data)
+        body = self._request('copy', {'context': self.ctx, 'data': data})
+        self.assertIsNone(body.get('error'))
+        node = self.serializer.deserialize_entity(self.ctx, body['result'])
         logged_params = mock_log.call_args_list[0][0][2]
-        logged_node = logged_params['node']['ironic_object.data']
-        self.assertEqual('***', logged_node['driver_info']['ipmi_password'])
+        logged_node = logged_params['data']
+        self.assertEqual({'ipmi_username': 'admin', 'ipmi_password': '***'},
+                         logged_node)
         logged_resp = mock_log.call_args_list[1][0][2]
-        logged_node = logged_resp['ironic_object.data']
-        self.assertEqual('***', logged_node['driver_info']['ipmi_password'])
+        self.assertEqual({'ipmi_username': 'admin', 'ipmi_password': '***'},
+                         logged_resp)
         # The result is not affected, only logging
-        self.assertEqual(db_utils.get_test_ipmi_info(), node.driver_info)
+        self.assertEqual(data, node)
 
 
 @mock.patch.object(client, '_get_session', autospec=True)
-class TestClient(test_base.TestCase):
+class TestClient(TestCase):
 
     def setUp(self):
         super(TestClient, self).setUp()
-        self.serializer = objects_base.IronicObjectSerializer(is_server=True)
+        self.serializer = FakeSerializer()
         self.client = client.Client(self.serializer)
+        self.context = FakeContext({'user_name': 'admin'})
         self.ctx_json = self.context.to_dict()
 
     def test_can_send_version(self, mock_session):
@@ -348,18 +370,70 @@ class TestClient(test_base.TestCase):
                   'params': {'answer': 42, 'context': self.ctx_json},
                   'id': self.context.request_id})
 
+    def test_call_ipv4_success(self, mock_session):
+        response = mock_session.return_value.post.return_value
+        response.json.return_value = {
+            'jsonrpc': '2.0',
+            'result': 42
+        }
+        cctx = self.client.prepare('foo.192.0.2.1')
+        self.assertEqual('192.0.2.1', cctx.host)
+        result = cctx.call(self.context, 'do_something', answer=42)
+        self.assertEqual(42, result)
+        mock_session.return_value.post.assert_called_once_with(
+            'http://192.0.2.1:8089',
+            json={'jsonrpc': '2.0',
+                  'method': 'do_something',
+                  'params': {'answer': 42, 'context': self.ctx_json},
+                  'id': self.context.request_id})
+
+    def test_call_ipv6_success(self, mock_session):
+        response = mock_session.return_value.post.return_value
+        response.json.return_value = {
+            'jsonrpc': '2.0',
+            'result': 42
+        }
+        cctx = self.client.prepare('foo.2001:db8::1')
+        self.assertEqual('2001:db8::1', cctx.host)
+        self.assertEqual(8089, cctx.port)
+        result = cctx.call(self.context, 'do_something', answer=42)
+        self.assertEqual(42, result)
+        mock_session.return_value.post.assert_called_once_with(
+            'http://[2001:db8::1]:8089',
+            json={'jsonrpc': '2.0',
+                  'method': 'do_something',
+                  'params': {'answer': 42, 'context': self.ctx_json},
+                  'id': self.context.request_id})
+
+    def test_call_ipv6_success_rfc2732(self, mock_session):
+        response = mock_session.return_value.post.return_value
+        response.json.return_value = {
+            'jsonrpc': '2.0',
+            'result': 42
+        }
+        cctx = self.client.prepare('foo.[2001:db8::1]:8192')
+        self.assertEqual('2001:db8::1', cctx.host)
+        result = cctx.call(self.context, 'do_something', answer=42)
+        self.assertEqual(42, result)
+        mock_session.return_value.post.assert_called_once_with(
+            'http://[2001:db8::1]:8192',
+            json={'jsonrpc': '2.0',
+                  'method': 'do_something',
+                  'params': {'answer': 42, 'context': self.ctx_json},
+                  'id': self.context.request_id})
+
     def test_call_success_with_version(self, mock_session):
         response = mock_session.return_value.post.return_value
         response.json.return_value = {
             'jsonrpc': '2.0',
             'result': 42
         }
-        cctx = self.client.prepare('foo.example.com', version='1.42')
+        cctx = self.client.prepare('foo.example.com:8192', version='1.42')
         self.assertEqual('example.com', cctx.host)
         result = cctx.call(self.context, 'do_something', answer=42)
         self.assertEqual(42, result)
         mock_session.return_value.post.assert_called_once_with(
-            'http://example.com:8089',
+            'http://example.com:8192',
             json={'jsonrpc': '2.0',
                   'method': 'do_something',
                   'params': {'answer': 42, 'context': self.ctx_json,
@@ -385,6 +459,24 @@ class TestClient(test_base.TestCase):
                              'rpc.version': '1.42'},
                   'id': self.context.request_id})
 
+    def test_call_with_ssl(self, mock_session):
+        self.config(use_ssl=True, group='json_rpc')
+        response = mock_session.return_value.post.return_value
+        response.json.return_value = {
+            'jsonrpc': '2.0',
+            'result': 42
+        }
+        cctx = self.client.prepare('foo.example.com')
+        self.assertEqual('example.com', cctx.host)
+        result = cctx.call(self.context, 'do_something', answer=42)
+        self.assertEqual(42, result)
+        mock_session.return_value.post.assert_called_once_with(
+            'https://example.com:8089',
+            json={'jsonrpc': '2.0',
+                  'method': 'do_something',
+                  'params': {'answer': 42, 'context': self.ctx_json},
+                  'id': self.context.request_id})
+
     def test_cast_success(self, mock_session):
         cctx = self.client.prepare('foo.example.com')
         self.assertEqual('example.com', cctx.host)
@@ -408,26 +500,6 @@ class TestClient(test_base.TestCase):
                   'params': {'answer': 42, 'context': self.ctx_json,
                              'rpc.version': '1.42'}})
 
-    def test_call_serialization(self, mock_session):
-        node = obj_utils.get_test_node(self.context)
-        node_json = self.serializer.serialize_entity(self.context, node)
-        response = mock_session.return_value.post.return_value
-        response.json.return_value = {
-            'jsonrpc': '2.0',
-            'result': node_json
-        }
-        cctx = self.client.prepare('foo.example.com')
-        self.assertEqual('example.com', cctx.host)
-        result = cctx.call(self.context, 'do_something', node=node)
-        self.assertIsInstance(result, objects.Node)
-        self.assertEqual(result.uuid, node.uuid)
-        mock_session.return_value.post.assert_called_once_with(
-            'http://example.com:8089',
-            json={'jsonrpc': '2.0',
-                  'method': 'do_something',
-                  'params': {'node': node_json, 'context': self.ctx_json},
-                  'id': self.context.request_id})
-
     def test_call_failure(self, mock_session):
         response = mock_session.return_value.post.return_value
         response.json.return_value = {
@@ -443,7 +515,7 @@ class TestClient(test_base.TestCase):
         cctx = self.client.prepare('foo.example.com')
         self.assertEqual('example.com', cctx.host)
         # Make sure that the class is restored correctly for expected errors.
-        exc = self.assertRaises(exception.Invalid,
+        exc = self.assertRaises(exception.BadRequest,
                                 cctx.call,
                                 self.context, 'do_something', answer=42)
         # Code from the body has priority over one in the class.
@@ -566,15 +638,15 @@ class TestClient(test_base.TestCase):
                   'method': 'do_something',
                   'params': {'node': request, 'context': self.ctx_json}})
         self.assertEqual(2, mock_log.call_count)
-        node = mock_log.call_args_list[0][0][2]['params']['node']
+        node = mock_log.call_args_list[0][0][3]['params']['node']
         self.assertEqual(node, {'redfish_username': 'admin',
                                 'redfish_password': '***'})
-        resp_text = mock_log.call_args_list[1][0][2]
+        resp_text = mock_log.call_args_list[1][0][3]
         self.assertEqual(body.replace('passw0rd', '***'), resp_text)
 
 
 @mock.patch('ironic.common.json_rpc.client.keystone', autospec=True)
-class TestSession(test_base.TestCase):
+class TestSession(TestCase):
 
     def setUp(self):
         super(TestSession, self).setUp()
@@ -584,9 +656,11 @@ class TestSession(test_base.TestCase):
         self.config(auth_strategy='noauth', group='json_rpc')
         session = client._get_session()
 
-        mock_keystone.get_auth.assert_not_called()
+        mock_keystone.get_auth.assert_called_once_with('json_rpc')
+        auth = mock_keystone.get_auth.return_value
+
         mock_keystone.get_session.assert_called_once_with(
-            'json_rpc', auth=None)
+            'json_rpc', auth=auth)
 
         internal_session = mock_keystone.get_session.return_value
 
@@ -620,13 +694,34 @@ class TestSession(test_base.TestCase):
 
     def test_http_basic(self, mock_keystone):
         self.config(auth_strategy='http_basic', group='json_rpc')
-        self.config(http_basic_username='myName', group='json_rpc')
-        self.config(http_basic_password='myPassword', group='json_rpc')
         session = client._get_session()
 
-        mock_keystone.get_auth.assert_not_called()
+        mock_keystone.get_auth.assert_called_once_with('json_rpc')
+        auth = mock_keystone.get_auth.return_value
         mock_keystone.get_session.assert_called_once_with(
-            'json_rpc', auth=None)
+            'json_rpc', auth=auth)
+
+        internal_session = mock_keystone.get_session.return_value
+
+        mock_keystone.get_adapter.assert_called_once_with(
+            'json_rpc',
+            session=internal_session,
+            additional_headers={
+                'Content-Type': 'application/json'
+            })
+        self.assertEqual(mock_keystone.get_adapter.return_value, session)
+
+    def test_http_basic_deprecated(self, mock_keystone):
+        self.config(auth_strategy='http_basic', group='json_rpc')
+        self.config(http_basic_username='myName', group='json_rpc')
+        self.config(http_basic_password='myPassword', group='json_rpc')
+        session = client._get_session()
+
+        mock_keystone.get_auth.assert_called_once_with(
+            'json_rpc', username='myName', password='myPassword')
+        auth = mock_keystone.get_auth.return_value
+        mock_keystone.get_session.assert_called_once_with(
+            'json_rpc', auth=auth)
 
         internal_session = mock_keystone.get_session.return_value
 
@@ -634,7 +729,6 @@ class TestSession(test_base.TestCase):
             'json_rpc',
             session=internal_session,
             additional_headers={
-                'Authorization': 'Basic bXlOYW1lOm15UGFzc3dvcmQ=',
                 'Content-Type': 'application/json'
             })
         self.assertEqual(mock_keystone.get_adapter.return_value, session)
