@@ -27,11 +27,13 @@ from ironic.objects import chassis
 from ironic.objects import conductor
 from ironic.objects import deploy_template
 from ironic.objects import firmware
+from ironic.objects import inspection_rule as rule
 from ironic.objects import node
 from ironic.objects import node_history
 from ironic.objects import node_inventory
 from ironic.objects import port
 from ironic.objects import portgroup
+from ironic.objects import runbook
 from ironic.objects import trait
 from ironic.objects import volume_connector
 from ironic.objects import volume_target
@@ -234,6 +236,7 @@ def get_test_node(**kw):
         'shard': kw.get('shard', None),
         'parent_node': kw.get('parent_node', None),
         'service_step': kw.get('service_step'),
+        'disable_power_off': kw.get('disable_power_off', False),
     }
 
     for iface in drivers_base.ALL_INTERFACES:
@@ -673,6 +676,59 @@ def create_test_deploy_template(**kw):
     return dbapi.create_deploy_template(template)
 
 
+def get_test_runbook(**kw):
+    default_uuid = uuidutils.generate_uuid()
+    return {
+        'version': kw.get('version', runbook.Runbook.VERSION),
+        'created_at': kw.get('created_at'),
+        'updated_at': kw.get('updated_at'),
+        'id': kw.get('id', 234),
+        'name': kw.get('name', u'CUSTOM_DT1'),
+        'uuid': kw.get('uuid', default_uuid),
+        'steps': kw.get('steps', [get_test_runbook_step(
+            runbook_id=kw.get('id', 234))]),
+        'disable_ramdisk': kw.get('disable_ramdisk', False),
+        'extra': kw.get('extra', {}),
+        'public': kw.get('public', False),
+        'owner': kw.get('owner', None),
+    }
+
+
+def get_test_runbook_step(**kw):
+    return {
+        'created_at': kw.get('created_at'),
+        'updated_at': kw.get('updated_at'),
+        'id': kw.get('id', 345),
+        'runbook_id': kw.get('runbook_id', 234),
+        'interface': kw.get('interface', 'raid'),
+        'step': kw.get('step', 'create_configuration'),
+        'args': kw.get('args', {'logical_disks': []}),
+        'order': kw.get('order', 1)
+    }
+
+
+def create_test_runbook(**kw):
+    """Create a runbook in the DB and return Runbook model.
+
+    :param kw: kwargs with overriding values for the runbook.
+    :returns: Test Runbook DB object.
+    """
+    runbook = get_test_runbook(**kw)
+    dbapi = db_api.get_instance()
+    # Let DB generate an ID if one isn't specified explicitly.
+    if 'id' not in kw:
+        del runbook['id']
+    if 'steps' not in kw:
+        for step in runbook['steps']:
+            del step['id']
+            del step['runbook_id']
+    else:
+        for kw_step, runbook_step in zip(kw['steps'], runbook['steps']):
+            if 'id' not in kw_step:
+                del runbook_step['id']
+    return dbapi.create_runbook(runbook)
+
+
 def get_test_history(**kw):
     return {
         'id': kw.get('id', 345),
@@ -767,3 +823,60 @@ def get_test_firmware_component_list():
         {'component': 'BIOS', 'initial_version': 'v1.5.0',
          'current_version': 'v1.5.0', 'last_version_flashed': None},
     ]
+
+
+def get_test_inspection_rule(**kw):
+    default_uuid = uuidutils.generate_uuid()
+    return {
+        'version': kw.get('version', rule.InspectionRule.VERSION),
+        'uuid': kw.get('uuid', default_uuid),
+        'description': kw.get('description', 'an inspection rule'),
+        'sensitive': kw.get('sensitive', False),
+        'phase': kw.get('phase', 'main'),
+        'priority': kw.get('priority', 0),
+        'actions': kw.get('actions', [get_test_inspection_rule_action(
+            inspection_rule_id=kw.get('uuid'))]),
+        'conditions': kw.get('conditions', [
+            get_test_inspection_rule_condition(
+                inspection_rule_id=kw.get('uuid'))]),
+    }
+
+
+def get_test_inspection_rule_action(**kw):
+    action = {
+        'op': kw.get('op', 'set-attribute'),
+        'args': kw.get('args', ["/driver", "idrac"]),
+    }
+
+    if 'id' in kw:
+        action['id'] = kw['id']
+
+    action.update({k: v for k, v in kw.items() if k not in action})
+    return action
+
+
+def get_test_inspection_rule_condition(**kw):
+    condition = {
+        'op': kw.get('op', 'is-true'),
+        'args': kw.get('args', ["{node.auto_discovered}"]),
+        'multiple': kw.get('multiple', 'any'),
+    }
+
+    if 'id' in kw:
+        condition['id'] = kw['id']
+
+    condition.update({k: v for k, v in kw.items() if k not in condition})
+    return condition
+
+
+def create_test_inspection_rule(**kw):
+    """Create a inspection rule in the DB and return InspectionRule model.
+
+    :param kw: kwargs with overriding values for the inspection rule.
+    :returns: Test InspectionRule DB object.
+    """
+    inspection_rule = get_test_inspection_rule(**kw)
+    dbapi = db_api.get_instance()
+    if 'uuid' not in kw:
+        del inspection_rule['uuid']
+    return dbapi.create_inspection_rule(inspection_rule)
