@@ -76,10 +76,12 @@ class RedfishPowerTestCase(db_base.DbTestCase):
                 mock_get_system.assert_called_once_with(task.node)
                 mock_get_system.reset_mock()
 
+    @mock.patch('time.sleep', autospec=True)
     @mock.patch.object(redfish_mgmt.RedfishManagement, 'restore_boot_device',
                        autospec=True)
     @mock.patch.object(redfish_utils, 'get_system', autospec=True)
-    def test_set_power_state(self, mock_get_system, mock_restore_bootdev):
+    def test_set_power_state(self, mock_get_system, mock_restore_bootdev,
+                             mock_sleep):
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
             expected_values = [
@@ -89,8 +91,10 @@ class RedfishPowerTestCase(db_base.DbTestCase):
                 (states.SOFT_REBOOT, sushy.RESET_GRACEFUL_RESTART, True),
                 (states.SOFT_POWER_OFF, sushy.RESET_GRACEFUL_SHUTDOWN, False)
             ]
+            mock_sleep.reset_mock()
 
             for target, expected, restore_bootdev in expected_values:
+                sleeps = target == states.REBOOT
                 if target in (states.POWER_OFF, states.SOFT_POWER_OFF):
                     final = sushy.SYSTEM_POWER_STATE_OFF
                     transient = sushy.SYSTEM_POWER_STATE_ON
@@ -114,13 +118,19 @@ class RedfishPowerTestCase(db_base.DbTestCase):
                         task.driver.management, task, system_result[0])
                 else:
                     self.assertFalse(mock_restore_bootdev.called)
+                if sleeps:
+                    mock_sleep.assert_called_with(15)
+                else:
+                    mock_sleep.assert_not_called()
 
                 # Reset mocks
                 mock_get_system.reset_mock()
                 mock_restore_bootdev.reset_mock()
+                mock_sleep.reset_mock()
 
+    @mock.patch('time.sleep', autospec=True)
     @mock.patch.object(redfish_utils, 'get_system', autospec=True)
-    def test_set_power_state_not_reached(self, mock_get_system):
+    def test_set_power_state_not_reached(self, mock_get_system, mock_sleep):
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
             self.config(power_state_change_timeout=2, group='conductor')
@@ -193,6 +203,37 @@ class RedfishPowerTestCase(db_base.DbTestCase):
             mock_restore_bootdev.assert_called_once_with(
                 task.driver.management, task, system_result[0])
 
+    @mock.patch.object(redfish_mgmt.RedfishManagement, 'restore_boot_device',
+                       autospec=True)
+    @mock.patch.object(redfish_utils, 'get_system', autospec=True)
+    def test_reboot_from_power_off_with_disable_power_off(
+            self, mock_get_system, mock_restore_bootdev):
+        # NOTE(dtantsur): if a node with disable_power_off is powered off, we
+        # probably cannot do anything about it. This unit test is only here
+        # for consistent coverage.
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            task.node.disable_power_off = True
+            system_result = [
+                # Initial state
+                mock.Mock(power_state=sushy.SYSTEM_POWER_STATE_OFF),
+                # Transient state - still powered off
+                mock.Mock(power_state=sushy.SYSTEM_POWER_STATE_OFF),
+                # Final state - down powering off
+                mock.Mock(power_state=sushy.SYSTEM_POWER_STATE_ON)
+            ]
+            mock_get_system.side_effect = system_result
+
+            task.driver.power.reboot(task)
+
+            # Asserts
+            system_result[0].reset_system.assert_called_once_with(
+                sushy.RESET_ON)
+            mock_get_system.assert_called_with(task.node)
+            self.assertEqual(3, mock_get_system.call_count)
+            mock_restore_bootdev.assert_called_once_with(
+                task.driver.management, task, system_result[0])
+
     @mock.patch.object(redfish_mgmt.RedfishManagement, 'restore_boot_device',
                        autospec=True)
     @mock.patch.object(redfish_utils, 'get_system', autospec=True)
@@ -221,6 +262,36 @@ class RedfishPowerTestCase(db_base.DbTestCase):
             mock_restore_bootdev.assert_called_once_with(
                 task.driver.management, task, system_result[0])
 
+    @mock.patch('time.sleep', autospec=True)
+    @mock.patch.object(redfish_mgmt.RedfishManagement, 'restore_boot_device',
+                       autospec=True)
+    @mock.patch.object(redfish_utils, 'get_system', autospec=True)
+    def test_reboot_from_power_on_with_disable_power_off(
+            self, mock_get_system, mock_restore_bootdev, mock_sleep):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            task.node.disable_power_off = True
+            system_result = [
+                # Initial state
+                mock.Mock(power_state=sushy.SYSTEM_POWER_STATE_ON),
+                # Transient state - powering off
+                mock.Mock(power_state=sushy.SYSTEM_POWER_STATE_OFF),
+                # Final state - down powering off
+                mock.Mock(power_state=sushy.SYSTEM_POWER_STATE_ON)
+            ]
+            mock_get_system.side_effect = system_result
+
+            task.driver.power.reboot(task)
+
+            # Asserts
+            system_result[0].reset_system.assert_called_once_with(
+                sushy.RESET_FORCE_RESTART)
+            mock_get_system.assert_called_with(task.node)
+            self.assertEqual(3, mock_get_system.call_count)
+            mock_restore_bootdev.assert_called_once_with(
+                task.driver.management, task, system_result[0])
+            mock_sleep.assert_called_with(15)
+
     @mock.patch.object(redfish_utils, 'get_system', autospec=True)
     def test_reboot_not_reached(self, mock_get_system):
         with task_manager.acquire(self.context, self.node.uuid,
