@@ -12,6 +12,7 @@
 
 from unittest import mock
 
+from ironic.db.sqlalchemy.api import Connection as db_conn
 from ironic import objects
 from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.db import utils as db_utils
@@ -25,25 +26,26 @@ class TestNodeInventoryObject(db_base.DbTestCase, obj_utils.SchemasTestMixIn):
         self.fake_inventory = db_utils.get_test_inventory()
 
     def test_create(self):
-        with mock.patch.object(self.dbapi, 'create_node_inventory',
+        with mock.patch.object(db_conn, 'create_node_inventory',
                                autospec=True) as mock_db_create:
             mock_db_create.return_value = self.fake_inventory
             new_inventory = objects.NodeInventory(
                 self.context, **self.fake_inventory)
             new_inventory.create()
 
-            mock_db_create.assert_called_once_with(self.fake_inventory)
+            mock_db_create.assert_called_once_with(mock.ANY,
+                                                   self.fake_inventory)
 
     def test_destroy(self):
         node_id = self.fake_inventory['node_id']
         with mock.patch.object(self.dbapi, 'get_node_inventory_by_node_id',
                                autospec=True) as mock_get:
             mock_get.return_value = self.fake_inventory
-            with mock.patch.object(self.dbapi,
+            with mock.patch.object(db_conn,
                                    'destroy_node_inventory_by_node_id',
                                    autospec=True) as mock_db_destroy:
                 inventory = objects.NodeInventory.get_by_node_id(self.context,
                                                                  node_id)
                 inventory.destroy()
 
-                mock_db_destroy.assert_called_once_with(node_id)
+                mock_db_destroy.assert_called_once_with(mock.ANY, node_id)
